# -*- coding: latin-1 -*-
import datetime

from django.db import models

class Credit(models.Model):
  amount = models.FloatField()
  acceptor = models.ForeignKey('BillAcceptor', blank=True, null=True)
  user = models.ForeignKey('auth.User', related_name='credits', blank=True,
      null=True)
  date = models.DateTimeField(default=datetime.datetime.now)

  # Reference to funds source.
  source = models.CharField(max_length=128,
      choices = ( ('cash', 'Cash'),
                  ('credit', 'Owner Credit'),
                  ('charge', 'Credit Card'),
                  ('gcheckout', 'Google Checkout'),
      ),
      default = 'cash',
  )

  # Purely administrative field to indicate payment status.
  status = models.CharField(max_length=128,
      choices = ( ('pending', 'Pending'),
                  ('processing', 'Processing'),
                  ('complete', 'Complete'),
                  ('invalid', 'Invalid'),
                  ('cancelled', 'Cancelled'),
      ),
      default = 'complete',
  )


class BillAcceptor(models.Model):
  name = models.CharField(max_length=128)
  increment = models.FloatField(default=1.0)

  def __str__(self):
    return 'BillAcceptor "%s"' % (self.name,)



# Copyright 2003-2009 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import math

from pykeg.core import backend
from pykeg.core import models
from pykeg.core import units

from pykeg.beerdb import models as bdb

def db_is_installed():
  try:
    models.Config.objects.get(key='db.installed')
    return True
  except models.Config.DoesNotExist:
    return False

def set_defaults():
  """ default values (contents may change with schema) """
  if db_is_installed():
    raise RuntimeError, "Database is already installed."

  site = models.KegbotSite.objects.create(name='default')

  # config table defaults
  default_config = (
     ('logging.logfile', 'keg.log'),
     ('logging.logformat', '%(asctime)s %(levelname)-8s (%(name)s) %(message)s'),
     ('logging.use_logfile', 'true'),
     ('logging.use_stream', 'true'),
     ('db.installed', 'true'),
  )
  for key, val in default_config:
    rec = models.Config(site=site, key=key, value=val)
    rec.save()

  # KegTap defaults
  main_tap = models.KegTap(site=site, name='Main Tap', meter_name='kegboard.flow0')
  main_tap.save()
  secondary_tap = models.KegTap(site=site, name='Second Tap', meter_name='kegboard.flow1')
  secondary_tap.save()

  b = backend.KegbotBackend()

  # brewer defaults
  unk_brewer = bdb.Brewer(name='Unknown Brewer')
  unk_brewer.save()

  # beerstyle defaults
  unk_style = bdb.BeerStyle(name='Unknown Style')
  unk_style.save()

  # beertype defaults
  unk_type = bdb.BeerType(name="Unknown Beer", brewer=unk_brewer, style=unk_style)
  unk_type.save()

  # KegSize defaults - from http://en.wikipedia.org/wiki/Keg#Size
  default_sizes = (
    (15.5, "Full Keg (half barrel)"),
    (13.2, "Import Keg (European barrel)"),
    (7.75, "Pony Keg (quarter barrel)"),
    (6.6, "European Half Barrel"),
    (5.23, "Sixth Barrel (torpedo keg)"),
    (5.0, "Corny Keg"),
    (1.0, "Mini Keg"),
  )
  for gallons, description in default_sizes:
    volume = units.Quantity(gallons, units.UNITS.USGallon)
    volume_int = volume.Amount(in_units=units.RECORD_UNIT)

    ks = models.KegSize(
      name=description,
      volume_ml=volume_int,
    )
    ks.save()

def gentestdata():
  """ default values (contents may change with schema) """
  sn = bdb.Brewer(name='Sierra Nevada Brewing Company',
        country='USA',
        origin_state='California',
        origin_city='Chico',
        production='commercial',
        url='http://www.sierranevada.com/')
  sn.save()

  an = bdb.Brewer(name='Anchor Brewing Company',
        country='USA',
        origin_state='California',
        origin_city='San Francisco',
        production='commercial',
        url='http://www.anchorsteam.com/')
  an.save()

  # beerstyle defaults
  pale_ale = bdb.BeerStyle(name='Pale Ale')
  pale_ale.save()

  # beertype defaults
  sn_pa = bdb.BeerType(name="Sierra Nevada Pale Ale",
        brewer=sn,
        style=pale_ale,
        calories_oz=10,
        carbs_oz=10,
        abv=5.5)
  sn_pa.save()

  as_pa = bdb.BeerType(name="Anchor Liberty Ale",
        brewer=an,
        style=pale_ale,
        calories_oz=10,
        carbs_oz=10,
        abv=5.0)
  as_pa.save()

  usernames = ['abe', 'bort', 'charlie']
  users = []
  b = backend.KegbotBackend()
  for name in usernames:
    users.append(b.CreateNewUser(name))

  half_barrel = models.KegSize(name="half barrel", volume_ml=10000)
  half_barrel.save()

  k = models.Keg(type=as_pa, size=half_barrel, status='online',
                 origcost=100)
  k.save()

  drink_base = datetime.datetime(2007,1,1,8,0,0)
  drink_interval = datetime.timedelta(seconds=600)
  drink_num = 0
  drink_vols = []
  for ml in (2200, 1100, 550, 715, 780):
    drink_vols.append(units.Quantity(ml, from_units=units.UNITS.KbMeterTick))

  # generate some drinks
  times = (drink_base, drink_base + datetime.timedelta(days=1))

  for drink_time in times:
    for rounds in range(3):
      for u in users:
        start = drink_time + drink_num*drink_interval
        end = start + datetime.timedelta(seconds=10)
        vol = drink_vols[drink_num%len(drink_vols)]
        drink = models.Drink(ticks=vol.ConvertTo.KbMeterTick,
                             volume_ml=vol.Amount(units.RECORD_UNIT),
                             starttime=start, endtime=end, user=u, keg=k,
                             status='valid')
        drink.save()
        drink_num += 1

  # fake thermo data
  thermo_start = datetime.datetime.now() - datetime.timedelta(hours=24)
  sensor_name = "thermo-0000000000000000"
  sensor = models.ThermoSensor.objects.create(raw_name=sensor_name,
      nice_name='Test sensor')
  for minute in xrange(60*24):
    temp_time = thermo_start + datetime.timedelta(minutes=minute)
    slot = (minute + 1)/30.0
    var = math.cos(2 * math.pi * slot)
    temp_value = 5.0 + 2.0*var
    record = models.Thermolog(sensor=sensor, temp=temp_value, time=temp_time)
    record.save()



# -*- coding: latin-1 -*-

from south.db import db
from django.db import models
from pykeg.web.accounting.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'DrinkCharge'
        db.create_table('accounting_drinkcharge', (
            ('id', orm['accounting.DrinkCharge:id']),
            ('user', orm['accounting.DrinkCharge:user']),
            ('base_funds', orm['accounting.DrinkCharge:base_funds']),
            ('ext_funds', orm['accounting.DrinkCharge:ext_funds']),
            ('tax_funds', orm['accounting.DrinkCharge:tax_funds']),
            ('handling_funds', orm['accounting.DrinkCharge:handling_funds']),
            ('statement', orm['accounting.DrinkCharge:statement']),
            ('drink', orm['accounting.DrinkCharge:drink']),
        ))
        db.send_create_signal('accounting', ['DrinkCharge'])
        
        # Adding model 'MiscCharge'
        db.create_table('accounting_misccharge', (
            ('id', orm['accounting.MiscCharge:id']),
            ('user', orm['accounting.MiscCharge:user']),
            ('base_funds', orm['accounting.MiscCharge:base_funds']),
            ('ext_funds', orm['accounting.MiscCharge:ext_funds']),
            ('tax_funds', orm['accounting.MiscCharge:tax_funds']),
            ('handling_funds', orm['accounting.MiscCharge:handling_funds']),
            ('statement', orm['accounting.MiscCharge:statement']),
            ('short_desc', orm['accounting.MiscCharge:short_desc']),
            ('long_desc', orm['accounting.MiscCharge:long_desc']),
        ))
        db.send_create_signal('accounting', ['MiscCharge'])
        
        # Adding model 'BillStatement'
        db.create_table('accounting_billstatement', (
            ('id', orm['accounting.BillStatement:id']),
            ('user', orm['accounting.BillStatement:user']),
            ('statement_date', orm['accounting.BillStatement:statement_date']),
            ('statement_id', orm['accounting.BillStatement:statement_id']),
            ('short_desc', orm['accounting.BillStatement:short_desc']),
            ('long_desc', orm['accounting.BillStatement:long_desc']),
        ))
        db.send_create_signal('accounting', ['BillStatement'])
        
        # Adding model 'Payment'
        db.create_table('accounting_payment', (
            ('id', orm['accounting.Payment:id']),
            ('user', orm['accounting.Payment:user']),
            ('base_funds', orm['accounting.Payment:base_funds']),
            ('ext_funds', orm['accounting.Payment:ext_funds']),
            ('tax_funds', orm['accounting.Payment:tax_funds']),
            ('handling_funds', orm['accounting.Payment:handling_funds']),
            ('source_user', orm['accounting.Payment:source_user']),
            ('statement', orm['accounting.Payment:statement']),
            ('payment_date', orm['accounting.Payment:payment_date']),
            ('payment_funds_source', orm['accounting.Payment:payment_funds_source']),
            ('transaction_id', orm['accounting.Payment:transaction_id']),
            ('payment_status', orm['accounting.Payment:payment_status']),
            ('status_message', orm['accounting.Payment:status_message']),
        ))
        db.send_create_signal('accounting', ['Payment'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'DrinkCharge'
        db.delete_table('accounting_drinkcharge')
        
        # Deleting model 'MiscCharge'
        db.delete_table('accounting_misccharge')
        
        # Deleting model 'BillStatement'
        db.delete_table('accounting_billstatement')
        
        # Deleting model 'Payment'
        db.delete_table('accounting_payment')
        
    
    
    models = {
        'accounting.billstatement': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'long_desc': ('django.db.models.fields.TextField', [], {}),
            'short_desc': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'statement_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'statement_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '128'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'accounting.drinkcharge': {
            'base_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'drink': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Drink']"}),
            'ext_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'handling_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'statement': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'drink_charges'", 'null': 'True', 'to': "orm['accounting.BillStatement']"}),
            'tax_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'accounting.misccharge': {
            'base_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'ext_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'handling_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'long_desc': ('django.db.models.fields.TextField', [], {}),
            'short_desc': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'statement': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'misc_charges'", 'null': 'True', 'to': "orm['accounting.BillStatement']"}),
            'tax_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'accounting.payment': {
            'base_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'ext_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'handling_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'payment_date': ('django.db.models.fields.DateTimeField', [], {}),
            'payment_funds_source': ('django.db.models.fields.CharField', [], {'default': "'cash'", 'max_length': '128'}),
            'payment_status': ('django.db.models.fields.CharField', [], {'default': "'complete'", 'max_length': '128'}),
            'source_user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'outgoing_payments'", 'to': "orm['auth.User']"}),
            'statement': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['accounting.BillStatement']", 'null': 'True', 'blank': 'True'}),
            'status_message': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'tax_funds': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'transaction_id': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'auth.group': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)"},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'core.beerstyle': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'core.beertype': {
            'abv': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'brewer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Brewer']"}),
            'calories_oz': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'carbs_oz': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'style': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.BeerStyle']"})
        },
        'core.brewer': {
            'comment': ('django.db.models.fields.TextField', [], {'default': "''", 'null': 'True', 'blank': 'True'}),
            'distribution': ('django.db.models.fields.CharField', [], {'default': "'unknown'", 'max_length': '128'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'origin_city': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'origin_country': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128'}),
            'origin_state': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128'}),
            'url': ('django.db.models.fields.URLField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'core.drink': {
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keg': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Keg']", 'null': 'True', 'blank': 'True'}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'valid'", 'max_length': '128'}),
            'ticks': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"}),
            'volume': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        'core.keg': {
            'channel': ('django.db.models.fields.IntegerField', [], {}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'enddate': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'origcost': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'size': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.KegSize']"}),
            'startdate': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.BeerType']"})
        },
        'core.kegsize': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'volume': ('django.db.models.fields.IntegerField', [], {})
        }
    }
    
    complete_apps = ['accounting']



#!/usr/bin/env python
#
# Copyright 2009 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Phidget RFID monitor authentication module."""

from pykeg.core import importhacks

import gflags
import logging
import os
import time

from Phidgets.Devices import RFID
from Phidgets.PhidgetException import PhidgetException

from pykeg.core import kb_app
from pykeg.core import kb_common
from pykeg.core import util
from pykeg.core.net import kegnet


gflags.DEFINE_integer('phidget_wait_seconds', 0,
    'Wait this many seconds for a phidget device on startup before aborting. '
    'A value of 0 will cause startup to block until a device has '
    'been found.',
    lower_bound=0)

FLAGS = gflags.FLAGS


class RfidEventHandler:
  """Collection of Phidget RFID event handlers to report status to kegnet."""

  def __init__(self, client, rfid):
    self._kegnet_client = client
    self._rfid = rfid
    self._logger = logging.getLogger('rfid-handler')
    self._InstallHandlers(self._rfid)

  def _InstallHandlers(self, rfid):
    rfid.setOnAttachHandler(self.onRfidAttached)
    rfid.setOnDetachHandler(self.onRfidDetached)
    rfid.setOnErrorhandler(self.onRfidError)
    rfid.setOnOutputChangeHandler(self.onRfidOutputChange)
    rfid.setOnTagHandler(self.onRfidTagAdded)
    rfid.setOnTagLostHandler(self.onRfidTagRemoved)

  def onRfidAttached(self, event):
    self._logger.info('RFID device attached: %s' % event.device.getSerialNum())
    self._rfid.setAntennaOn(True)

  def onRfidDetached(self, event):
    self._logger.info('RFID device detached: %s' % event.device.getSerialNum())

  def onRfidError(self, event):
    self._logger.warning('Phidget error %i: %s' % (event.eCode, event.description))

  def onRfidOutputChange(self, event):
    self._logger.info('Output %i change: %s' % (event.index, event.state))

  def onRfidTagAdded(self, event):
    self._logger.info('RFID added: %s' % event.tag)
    self._rfid.setLEDOn(1)
    strval = str(event.tag).lower()
    self._kegnet_client.SendAuthTokenAdd(FLAGS.tap_name,
        kb_common.AUTH_MODULE_CONTRIB_PHIDGET_RFID, strval)

  def onRfidTagRemoved(self, event):
    self._logger.info('RFID removed: %s' % event.tag)
    self._rfid.setLEDOn(0)
    strval = str(event.tag).lower()
    self._kegnet_client.SendAuthTokenRemove(FLAGS.tap_name,
        kb_common.AUTH_MODULE_CONTRIB_PHIDGET_RFID, strval)


class RfidAuthenticationApp(kb_app.App):

  def _Setup(self):
    kb_app.App._Setup(self)

    self._logger.info('Setting up kegnet client ...')
    self._client = kegnet.KegnetClient()

    self._asyncore_thread = util.AsyncoreThread('asyncore')
    self._AddAppThread(self._asyncore_thread)

    self._logger.info('Setting up RFID controller ...')
    self._rfid = RFID.RFID()
    self._rfid_handler = RfidEventHandler(self._client, self._rfid)

  def _MainLoop(self):
    self._client.Reconnect()
    self._logger.info('Opening phidget connection ...')
    self._rfid.openPhidget()
    self._logger.info('Waiting for RFID device ...')
    try:
      self._rfid.waitForAttach(FLAGS.phidget_wait_seconds * 1000)
    except PhidgetException:
      self._logger.error('No device found after %i seconds!' %
          FLAGS.phidget_wait_seconds)
      return 1
    self._logger.info('Waiting for RFID events ...')
    while not self._do_quit:
      try:
        self._client.Reconnect()
        time.sleep(1.0)
      except KeyboardInterrupt:
        self._logger.info('Got CTRL-C, exiting.')
        break
    self._rfid.closePhidget()
    self._logger.info('Closed RFID device, exiting ...')


if __name__ == '__main__':
  RfidAuthenticationApp.BuildAndRun()




#!/usr/bin/env python
#
# Copyright 2009 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegbot Application module.

Module for common logic for a command line or daemon application.
"""

import logging
import os
import signal
import sys
import threading
import time
import traceback

import gflags

from pykeg.core import util

FLAGS = gflags.FLAGS

gflags.DEFINE_boolean('daemon', False,
    'Run application in daemon mode')

gflags.DEFINE_string('pidfile', '',
    "If specified, logs the application's process id to this file.")

gflags.DEFINE_string('logformat',
    '%(asctime)s %(levelname)-8s (%(name)s) %(message)s',
    'Default format to use for log messages.')

gflags.DEFINE_string('logfile',
    '%s.log' % os.path.basename(sys.argv[0].replace('.py','')),
    'Default log file for log messages')

gflags.DEFINE_boolean('log_to_file', False,
    'Send log messages to the log file defined by --logfile')

gflags.DEFINE_boolean('log_to_stdout', True,
    'Send log messages to the console')

gflags.DEFINE_boolean('verbose', os.environ.get('VERBOSE') is not None,
    'Generate extra logging information.')


class App(object):
  """Application instance container.

  Scripts and daemons wishing to reuse the application logic here should
  subclass App, and implement some or all of _Setup and _MainLoop.
  """

  def __init__(self, name='main', daemon=None):
    self._name = name

    try:
      self._extra_argv = FLAGS(sys.argv)  # parse flags
    except gflags.FlagsError, e:
      print 'Usage: %s ARGS\n%s\n\nError: %s' % (sys.argv[0], FLAGS, e)
      sys.exit(1)

    if daemon is None:
      daemon = FLAGS.daemon

    self._is_daemon = daemon
    self._do_quit = False
    self._quit_event = threading.Event()
    self._threads = set()

    self._SetupLogging()
    self._SetupSignalHandlers()
    self._logger = logging.getLogger(self._name)

  @classmethod
  def BuildAndRun(cls, name='main'):
    """Convenience class method to create and Start the app."""
    if sys.version_info < (2,4):
      print>>sys.stderr, 'kegbot requires Python 2.4 or later; aborting'
      sys.exit(1)

    try:
      app = cls(name)
      app.Start()
    except Exception, e:
      logger = lambda msg: sys.stderr.write('%s\n' % msg)
      logger('Uncaught exception in app %s. Stack trace:' % name)
      util.LogTraceback(logger)
      logger('Aborting.')
      sys.exit(1)

  def Start(self):
    """Perform setup and run the application main loop."""
    self._Setup()
    self._CheckAndRecordPid()
    self._StartThreads()
    self._MainLoop()
    self._Teardown()

  def _Setup(self):
    """Perform app-specific setup.

    This function is called by Start. Subclasses should call the superclass
    _Setup method (or provide similar functionality.
    """
    if self._is_daemon:
      self._logger.info('Daemon mode requested, switching to background.')
      util.daemonize()
      self._logger.info('Running in background.')

  def _CheckAndRecordPid(self):
    if not FLAGS.pidfile:
      return
    my_pid = os.getpid()
    self._logger.info('Logging PID (%i) to file %s' % (my_pid,
        FLAGS.pidfile))
    if os.path.exists(FLAGS.pidfile):
      self._logger.warning('Pidfile already exists, checking liveness')
      try:
        pid_fd = open(FLAGS.pidfile)
        old_pid = int(pid_fd.readline().strip())
        pid_fd.close()
      except (IOError, ValueError):
        self._logger.error('Could not read pid, abort.')
        sys.exit(1)
      if util.PidIsAlive(old_pid):
        self._logger.error('Old pid %i is still alive, aborting' % old_pid)
        sys.exit(1)

    try:
      pid_fd = open(FLAGS.pidfile, 'w')
      pid_fd.write('%i\n' % my_pid)
      pid_fd.close()
    except IOError:
      self._logger.error('Error writing pid to %s' % FLAGS.pidfile)
      sys.exit(1)

  def _StartThreads(self):
    """Start any threading.Thread objects registered in _threads."""
    if not len(self._threads):
      return
    self._logger.info('Starting all service threads.')
    for thr in self._threads:
      self._logger.info('starting thread "%s"' % thr.getName())
      thr.start()
    self._logger.info('All threads started.')

  def _StopThreads(self):
    """Stop any threading.Thread objects registered in _threads."""
    if not len(self._threads):
      return
    self._logger.info('Stopping all service threads.')
    for thr in self._threads:
      if thr.isAlive():
        self._logger.info('stopping thread "%s"' % thr.getName())
        thr.Quit()
      if thr.isAlive():
        thr.join(2.0)
    self._logger.info('All service threads stopped.')

  def _MainLoop(self):
    """Run the (possibly app-specific) main loop."""
    self._logger.info('Running generic main loop (going to sleep).')
    while not self._do_quit:
      self._quit_event.wait(timeout=1.0)
    self._logger.info('Exiting main loop')

  def _Teardown(self):
    """Clean up before quitting."""
    if FLAGS.pidfile:
      os.unlink(FLAGS.pidfile)

  def _DumpStatus(self):
    self._logger.info('-------- Begin Status Dump --------')
    if len(self._threads):
      self._logger.info('Status of threads:')
    for thr in self._threads:
      if thr.isAlive():
        thrstatus = 'running'
      else:
        thrstatus = '   DEAD'
      self._logger.info('%s: %s' % (thr.getName(), thrstatus))
      lines = thr.GetStatus()
      if lines:
        for line in lines:
          self._logger.info(line)
        self._logger.info('')
    self._logger.info('-------- End Status Dump --------')

  def _AddAppThread(self, thr):
    """Add a threading.Thread to the set of registered threads.

    Threads added this way will be owned by the app, and automatically started
    and stopped with the main application.
    """
    self._threads.add(thr)

  def Quit(self):
    """Run the (possibly app-specific) Quit routines."""
    self._StopThreads()
    self._TeardownLogging()
    self._do_quit = True
    self._quit_event.set()

  def _SetupLogging(self, level=None):
    if level is None:
      if FLAGS.verbose:
        level = logging.DEBUG
      else:
        level = logging.INFO
    logging.root.setLevel(level)

    # add a file-output handler
    self._logging_file_handler = None
    if FLAGS.log_to_file:
      self._logging_file_handler = logging.FileHandler(FLAGS.logfile)
      formatter = logging.Formatter(FLAGS.logformat)
      self._logging_file_handler.setFormatter(formatter)
      logging.root.addHandler(self._logging_file_handler)

    # add tty handler
    self._logging_stdout_handler = None
    if FLAGS.log_to_stdout and not self._is_daemon:
      self._logging_stdout_handler = logging.StreamHandler(sys.stdout)
      formatter = logging.Formatter(FLAGS.logformat)
      self._logging_stdout_handler.setFormatter(formatter)
      logging.root.addHandler(self._logging_stdout_handler)

  def _TeardownLogging(self):
    if self._logging_file_handler:
      logging.root.removeHandler(self._logging_file_handler)
      self._logging_file_handler = None
    if self._logging_stdout_handler:
      logging.root.removeHandler(self._logging_stdout_handler)
      self._logging_stdout_handler = None

  def _SetupSignalHandlers(self):
    """Set up handlers for signals received by main process.

    Sets HUP, INT, QUIT, TERM to cause a quit;
    Sets USR1 to cause a status dump.
    """
    signal.signal(signal.SIGHUP, self._QuitSignalHandler)
    signal.signal(signal.SIGINT, self._QuitSignalHandler)
    signal.signal(signal.SIGQUIT, self._QuitSignalHandler)
    signal.signal(signal.SIGTERM, self._QuitSignalHandler)
    signal.signal(signal.SIGUSR1, self._DumpSignalHandler)

  def _QuitSignalHandler(self, signum, frame):
    """ All handled signals cause a quit """
    self._logger.info('Got signal %i' % signum)
    self.Quit()

  def _DumpSignalHandler(self, signum, frame):
    self._DumpStatus()



#!/usr/bin/env python

"""Unittest for manager module"""

import unittest

from pykeg.core import event
from pykeg.core import kb_common
from pykeg.core import manager

class _MockKegbotCore(object):
  pass

class FlowManagerTestCase(unittest.TestCase):
  def setUp(self):
    event_hub = event.EventHub()
    self.tap_manager = manager.TapManager("tap-manager", event_hub)
    self.flow_manager = manager.FlowManager("flow-manager", event_hub, self.tap_manager)
    self.flow_manager.RegisterDevice(name='flow0')

  def tearDown(self):
    self.flow_manager.UnregisterDevice(name='flow0')

  def testBasicMeterUse(self):
    """Create a new flow device, perform basic operations on it."""
    # Duplicate registration should cause an exception.
    self.assertRaises(manager.AlreadyRegisteredError,
                      self.flow_manager.RegisterDevice, 'flow0')

    # ...as should operations on an unknown device.
    self.assertRaises(manager.UnknownDeviceError,
                      self.flow_manager.GetDeviceVolume, 'flow_unknown')
    self.assertRaises(manager.UnknownDeviceError,
                      self.flow_manager.UpdateDeviceReading, 'flow_unknown', 123)

    # Our new device should have accumulated 0 volume thus far.
    reading = self.flow_manager.GetDeviceVolume(name='flow0')
    self.assertEqual(reading, 0L)

    # Report an instantaneous reading of 2000 ticks. Since this is the first
    # reading, this should cause no change in the device volume.
    self.flow_manager.UpdateDeviceReading(name='flow0', value=2000)
    reading = self.flow_manager.GetDeviceVolume(name='flow0')
    self.assertEqual(reading, 0L)

    # Report another instantaneous reading, which should now increment the flow
    self.flow_manager.UpdateDeviceReading(name='flow0', value=2100)
    reading = self.flow_manager.GetDeviceVolume(name='flow0')
    self.assertEqual(reading, 100L)

    # The FlowManager saves the last reading value; check it.
    last_reading = self.flow_manager.GetDeviceLastReading(name='flow0')
    self.assertEqual(last_reading, 2100)

    # Report a reading that is much larger than the last reading. Values larger
    # than the constant kb_common.MAX_METER_READING_DELTA should be ignored by
    # the FlowManager.
    illegal_delta = kb_common.MAX_METER_READING_DELTA + 100
    new_reading = last_reading + illegal_delta
    self.flow_manager.UpdateDeviceReading(name='flow0', value=new_reading)
    # The illegal update should not affect the volume.
    vol = self.flow_manager.GetDeviceVolume(name='flow0')
    self.assertEqual(vol, 100L)
    # The value of the last update should be recorded, however.
    last_reading = self.flow_manager.GetDeviceLastReading(name='flow0')
    self.assertEqual(last_reading, new_reading)

  def testOverflowHandling(self):
    first_reading = 2**32 - 100    # start with very large number
    second_reading = 2**32 - 50    # increment by 50
    overflow_reading = 10          # increment by 50+10 (overflow)

    self.flow_manager.UpdateDeviceReading('flow0', first_reading)
    curr_reading = self.flow_manager.GetDeviceVolume('flow0')
    self.assertEqual(curr_reading, 0)

    self.flow_manager.UpdateDeviceReading('flow0', second_reading)
    curr_reading = self.flow_manager.GetDeviceVolume('flow0')
    self.assertEqual(curr_reading, 50)

    self.flow_manager.UpdateDeviceReading('flow0', overflow_reading)
    curr_reading = self.flow_manager.GetDeviceVolume('flow0')
    self.assertEqual(curr_reading, 110)

  def testNoOverflow(self):
    self.flow_manager.UpdateDeviceReading('flow0', 0)
    curr_reading = self.flow_manager.GetDeviceVolume('flow0')
    self.assertEqual(curr_reading, 0)

    self.flow_manager.UpdateDeviceReading('flow0', 100)
    curr_reading = self.flow_manager.GetDeviceVolume('flow0')
    self.assertEqual(curr_reading, 100)

    self.flow_manager.UpdateDeviceReading('flow0', 10)
    curr_reading = self.flow_manager.GetDeviceVolume('flow0')
    self.assertEqual(curr_reading, 100)

    self.flow_manager.UpdateDeviceReading('flow0', 20)
    curr_reading = self.flow_manager.GetDeviceVolume('flow0')
    self.assertEqual(curr_reading, 110)

  def testActivityMonitoring(self):
    self.flow_manager.UpdateDeviceReading('flow0', 0, when=0)

    # initial idle time should be infinite (should return `now`)
    idle_time = self.flow_manager.GetDeviceIdleSeconds('flow0', now=1000)
    self.assertEqual(idle_time, 1000)

    self.flow_manager.UpdateDeviceReading('flow0', 10, when=10)
    self.flow_manager.UpdateDeviceReading('flow0', 30, when=15)

    idle_time = self.flow_manager.GetDeviceIdleSeconds('flow0', now=20)
    self.assertEqual(idle_time, 5)

    # Updating the device again with zero delta should not clear existing idle
    # time.
    self.flow_manager.UpdateDeviceReading('flow0', 30, when=30)
    idle_time = self.flow_manager.GetDeviceIdleSeconds('flow0', now=40)
    self.assertEqual(idle_time, 25)

    # Updating the device with negative delta should also leave idle time
    # unchanged.
    self.flow_manager.UpdateDeviceReading('flow0', 10, when=50)
    idle_time = self.flow_manager.GetDeviceIdleSeconds('flow0', now=60)
    self.assertEqual(idle_time, 45)


if __name__ == '__main__':
  unittest.main()



"""Unittests for pykeg.core.models"""

import datetime
import unittest

from pykeg.core import models
from pykeg.core import units

from pykeg.beerdb import models as bdb_models

class CoreModelsTestCase(unittest.TestCase):
  def setUp(self):
    self.brewer = bdb_models.Brewer.objects.create(
        name='Moonshine Beers',
        country='USA',
        origin_state='Anystate',
        origin_city='Bathtub',
        production='retail',
        url='http://example.com/',
        description='Pretty bad beers.',
    )

    self.beer_style = bdb_models.BeerStyle.objects.create(
        name='Porter',
    )

    self.beer_type = bdb_models.BeerType.objects.create(
        name='Moonshine Porter',
        brewer=self.brewer,
        style=self.beer_style,
        calories_oz=3.0,
        carbs_oz=10.0,
        abv=0.05,
    )

    self.keg_vol = units.UnitConverter.Convert(2.0, units.UNITS.Liter,
                                               units.RECORD_UNIT)
    self.keg_size = models.KegSize.objects.create(
        name='Tiny Keg',
        volume_ml=self.keg_vol,
    )

    self.keg = models.Keg.objects.create(
        type=self.beer_type,
        size=self.keg_size,
        startdate=datetime.datetime(2000, 4, 1),
        enddate=datetime.datetime(2000, 5, 1),
        status='online',
        description='Our first keg!',
        origcost=99.0,
    )

    self.user = models.User.objects.create(
        username='kb_tester',
    )

    self.user2 = models.User.objects.create(
        username='kb_tester2',
    )

  def tearDown(self):
    self.user.delete()
    self.user2.delete()
    self.keg.delete()
    del self.keg_vol
    self.beer_type.delete()
    self.beer_style.delete()
    self.brewer.delete()

  def testKegStuff(self):
    """Test basic keg relations that should always work."""
    self.assertEqual(self.keg.size.Volume().ConvertTo.Liter, 2.0)
    self.assertEqual(self.keg.type.brewer.name, "Moonshine Beers")

    self.assertEqual(self.keg.served_volume(), 0.0)
    self.assertEqual(self.keg.remaining_volume(), self.keg_vol)

  def testDrinkAccounting(self):
    vol = units.Quantity(1200)

    d = models.Drink.objects.create(
        ticks=vol.ConvertTo.KbMeterTick,
        volume_ml=vol.ConvertTo.KbMeterTick,
        starttime=datetime.datetime.now(),
        endtime=datetime.datetime.now(),
        user=self.user,
        keg=self.keg,
        status='valid',
    )

    self.assertEqual(self.keg.served_volume(), d.volume_ml)

  def testDrinkSessions(self):
    """ Checks for the DrinkingSession records. """
    u1 = self.user
    u2 = self.user2
    vol = units.Quantity(1200)

    drinks = {}
    base_time = datetime.datetime(2009,1,1,1,0,0)

    ticks = volume = vol.ConvertTo.KbMeterTick

    td_10m = datetime.timedelta(minutes=10)
    td_200m = datetime.timedelta(minutes=200)
    td_190m = td_200m - td_10m

    num_inital_groups = len(models.DrinkingSession.objects.all())

    drinks[u1] = (
        # t=0
        models.Drink.objects.create(
          ticks=ticks, volume_ml=volume, user=u1, keg=self.keg,
          starttime=base_time, endtime=base_time),
        # t=10
        models.Drink.objects.create(
          ticks=ticks, volume_ml=volume, user=u1, keg=self.keg,
          starttime=base_time+td_10m, endtime=base_time+td_10m),
        # t=200
        models.Drink.objects.create(
          ticks=ticks, volume_ml=volume, user=u1, keg=self.keg,
          starttime=base_time+td_200m, endtime=base_time+td_200m),
    )

    drinks[u2] = (
        # t=0
        models.Drink.objects.create(
          ticks=ticks, volume_ml=volume, user=u2, keg=self.keg,
          starttime=base_time, endtime=base_time),
        # t=200
        models.Drink.objects.create(
          ticks=ticks, volume_ml=volume, user=u2, keg=self.keg,
          starttime=base_time+td_200m, endtime=base_time+td_200m),
        # t=190
        models.Drink.objects.create(
          ticks=ticks, volume_ml=volume, user=u2, keg=self.keg,
          starttime=base_time+td_190m, endtime=base_time+td_190m),
    )

    u1_session_parts = u1.session_parts.all()
    self.assertEqual(len(u1_session_parts), 2)

    u2_session_parts = u2.session_parts.all()
    self.assertEqual(len(u2_session_parts), 2)

    # user1 session 1: should be 10 minutes long as created above
    u1_s1 = u1_session_parts[0].session
    self.assertEqual(u1_s1.starttime, base_time)
    self.assertEqual(u1_s1.starttime, drinks[u1][0].starttime)
    self.assertEqual(u1_s1.endtime, drinks[u1][1].endtime)
    self.assertEqual(u1_s1.endtime - u1_s1.starttime, td_10m)
    self.assertEqual(len(list(u1_s1.drinks.all())), 2)

    # user1 session 2: at time 200, 1 drink
    u1_s2 = u1_session_parts[1].session
    self.assertEqual(u1_s2.starttime, base_time+td_200m)
    self.assertEqual(u1_s2.endtime, base_time+td_200m)
    self.assertEqual(len(list(u1_s2.drinks.all())), 1)

    # user2 session2: drinks are added out of order to create this, ensure times
    # match
    u2_s2 = u2_session_parts[1].session
    self.assertEqual(u2_s2.starttime, base_time+td_190m)
    self.assertEqual(u2_s2.endtime, base_time+td_200m)
    self.assertEqual(len(list(u2_s2.drinks.all())), 2)

    # Now check DrinkingSessions were created correctly; there should be
    # two groups capturing all 4 sessions.
    all_groups = models.DrinkingSession.objects.all()[num_inital_groups:]
    self.assertEqual(len(all_groups), 2)

    self.assertEqual(all_groups[0].starttime, base_time)
    self.assertEqual(all_groups[0].endtime, base_time+td_10m)
    self.assertEqual(len(all_groups[0].GetSessions()), 2)

    self.assertEqual(all_groups[1].starttime, base_time+td_190m)
    self.assertEqual(all_groups[1].endtime, base_time+td_200m)
    self.assertEqual(len(all_groups[1].GetSessions()), 2)



#!/usr/bin/env python
#
# Copyright 2009 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegboard daemon.

The kegboard daemon is the primary interface between a kegboard devices and a
kegbot system.  The process is responsible for several tasks, including:
  - discovering kegboards available locally
  - connecting to the kegbot core and registering the individual boards
  - accumulating data if the kegbot core is offline

The kegboard daemon is compatible with any device that speaks the Kegboard
Serial Protocol. See http://kegbot.org/docs for the complete specification.

The daemon should run on any machine which is attached to kegboard hardware.

The daemon must connect to a Kegbot Core in order to publish data (such as flow
and temperature events).  This is a TCP connection, using the Kegnet Protocol to
exchange data.
"""

import Queue

import gflags
import serial
import time

from pykeg.core import importhacks
from pykeg.core import kb_app
from pykeg.core import kb_common
from pykeg.core import util
from pykeg.core.net import kegnet
from pykeg.hw.kegboard import kegboard

FLAGS = gflags.FLAGS

gflags.DEFINE_string('kegboard_name', 'kegboard',
    'Name of this kegboard that will be used when talking to the core. '
    'If you are using multiple kegboards, you will want to run different '
    'daemons with different names. Otherwise, the default is fine.')

gflags.DEFINE_boolean('show_messages', True,
    'Print all messages going to and from the kegboard. Useful for '
    'debugging.')

gflags.DEFINE_integer('required_firmware_version', 4,
    'Minimum firmware version required.  If the device has an older '
    'firmware version, the daemon will refuse to service it.  This '
    'value should probably not be changed.')

class KegboardKegnetClient(kegnet.SimpleKegnetClient):
  pass

class KegboardManagerApp(kb_app.App):
  def __init__(self, name='core'):
    kb_app.App.__init__(self, name)

  def _Setup(self):
    kb_app.App._Setup(self)

    self._client = KegboardKegnetClient()

    self._client_thr = kegnet.KegnetClientThread('kegnet', self._client)
    self._AddAppThread(self._client_thr)

    self._manager_thr = KegboardManagerThread('kegboard-manager',
        self._client)
    self._AddAppThread(self._manager_thr)

    self._device_io_thr = KegboardDeviceIoThread('device-io', self._manager_thr,
        FLAGS.kegboard_device, FLAGS.kegboard_speed)
    self._AddAppThread(self._device_io_thr)


class KegboardManagerThread(util.KegbotThread):
  """Manager of local kegboard devices."""

  def __init__(self, name, client):
    util.KegbotThread.__init__(self, name)
    self._client = client
    self._message_queue = Queue.Queue()

  def _DeviceName(self, base_name):
    return '%s.%s' % (FLAGS.kegboard_name, base_name)

  def PostDeviceMessage(self, device_name, device_message):
    """Receive a message from a device, for processing."""
    self._message_queue.put((device_name, device_message))

  def run(self):
    self._logger.info('Starting main loop.')
    initialized = False

    while not self._quit:
      try:
        device_name, device_message = self._message_queue.get(timeout=1.0)
      except Queue.Empty:
        continue

      if FLAGS.show_messages:
        self._logger.info('RX: %s' % str(device_message))
      self._HandleDeviceMessage(device_name, device_message)

    self._logger.info('Exiting main loop.')

  def _HandleDeviceMessage(self, device_name, msg):
    if isinstance(msg, kegboard.MeterStatusMessage):
      meter_name = self._DeviceName(msg.meter_name)
      curr_val = msg.meter_reading
      self._client.SendMeterUpdate(meter_name, curr_val)

    elif isinstance(msg, kegboard.TemperatureReadingMessage):
      sensor_name = self._DeviceName(msg.sensor_name)
      sensor_value = msg.sensor_reading
      self._client.SendThermoUpdate(sensor_name, sensor_value)

    elif isinstance(msg, kegboard.OnewirePresenceMessage):
      strval = '%016x' % msg.device_id
      if msg.status == 1:
        self._client.SendAuthTokenAdd(FLAGS.tap_name,
            kb_common.AUTH_MODULE_CORE_ONEWIRE, strval)
      else:
        self._client.SendAuthTokenRemove(FLAGS.tap_name,
            kb_common.AUTH_MODULE_CORE_ONEWIRE, strval)


class KegboardDeviceIoThread(util.KegbotThread):
  """Manages all device I/O.

  This thread continuously reads from attached kegboard devices and passes
  messages to the KegboardManagerThread.
  """
  def __init__(self, name, manager, device_path, device_speed):
    util.KegbotThread.__init__(self, name)
    self._manager = manager
    self._device_path = device_path
    self._device_speed = device_speed
    self._reader = None
    self._serial_fd = None

  def _SetupSerial(self):
    self._logger.info('Setting up serial port...')
    self._serial_fd = serial.Serial(self._device_path, self._device_speed)
    self._reader = kegboard.KegboardReader(self._serial_fd)

  def run(self):
    self._SetupSerial()
    try:
      self._MainLoop()
    finally:
      self._serial_fd.close()

  def Ping(self):
    ping_message = kegboard.PingCommand()
    self._reader.WriteMessage(ping_message)

  def _MainLoop(self):
    self._logger.info('Starting reader loop...')

    # Ping the board a couple of times before going into the listen loop.
    for i in xrange(2):
      self.Ping()

    initialized = False
    while not self._quit:
      msg = self._reader.GetNextMessage()

      # Check the reported firmware version. If it is not acceptable, then
      # drop all messages until it is updated.
      # TODO(mikey): kill the application when this happens? It isn't strictly
      # necessary, but is probably the most obvious way to get the point across.
      if isinstance(msg, kegboard.HelloMessage):
        version = msg.firmware_version
        if version >= FLAGS.required_firmware_version:
          if not initialized:
            self._logger.info('Found a Kegboard! Firmware version %i' % version)
            initialized = True
        else:
          self._logger.error('Attached kegboard firmware version (%s) is '
              'less than the required version (%s); please update this '
              'kegboard.' % (actual, required))
          self._logger.warning('Messages from this board will be ignored '
              'until it is updated.')
          initialized = False

      if not initialized:
        self.Ping()
        time.sleep(0.1)
        continue

      self._manager.PostDeviceMessage('kegboard', msg)
    self._logger.info('Reader loop ended.')


if __name__ == '__main__':
  KegboardManagerApp.BuildAndRun()



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import os
import random

from django.db import models
from django.contrib.auth.models import User

def sound_file_name(instance, filename):
  rand_salt = random.randrange(0xffff)
  new_filename = '%04x-%s' % (rand_salt, filename)
  return os.path.join('sounds', new_filename)


class SoundFile(models.Model):
  sound = models.FileField(upload_to=sound_file_name)
  title = models.CharField(max_length=128)
  active = models.BooleanField(default=True)

  def __str__(self):
    return '%s (%s)' % (self.title, self.sound)


class SoundEvent(models.Model):
  event_name = models.CharField(max_length=256)
  event_predicate = models.CharField(max_length=256, blank=True, null=True)
  soundfile = models.ForeignKey(SoundFile)
  user = models.ForeignKey(User, blank=True, null=True)

  def SoundTitle(self):
    return self.soundfile.title

  def __str__(self):
    return '%s(%s) -> %s' % (self.event_name, self.event_predicate,
        self.soundfile.title)



#!/usr/bin/env python
#
# Copyright 2008 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegweb main views."""

import datetime

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import Http404
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.decorators.cache import cache_page
from django.views.generic.list_detail import object_detail
from django.views.generic.list_detail import object_list
from django.views.generic.simple import redirect_to

from pykeg.core import models
from pykeg.core import units

from pykeg.web.kegweb import forms
from pykeg.web.kegweb import models as kegweb_models
from pykeg.web.kegweb import view_util

### main views

@cache_page(30)
def index(request):
  context = RequestContext(request)
  try:
    page = kegweb_models.Page.objects.get(name__exact='MainPage',
                                          status__exact='published')
  except kegweb_models.Page.DoesNotExist:
    page = None

  context['page_node'] = page
  context['taps'] = request.kbsite.kegtap_set.all()

  num_drinks = getattr(settings, 'KEGWEB_LAST_DRINKS_COUNT', 10)
  drinks = request.kbsite.drinks.valid().order_by('-endtime')
  drinks = drinks[:num_drinks]
  context['latest_drinks'] = drinks

  return render_to_response('index.html', context)

@cache_page(30)
def system_stats(request):
  context = RequestContext(request)
  all_drinks = request.kbsite.drinks.valid()
  context['top_volume_drinkers_alltime'] = view_util.drinkers_by_volume(all_drinks)[:10]
  return render_to_response('kegweb/system-stats.html', context)


### object lists and detail (generic views)

def user_list(request):
  user_list = models.User.objects.all()
  return object_list(request,
      queryset=user_list,
      template_object_name='drinker',
      template_name='kegweb/drinker_list.html')

def user_detail(request, username):
  return object_detail(request,
      queryset=models.User.objects.filter(is_active=True),
      slug=username,
      slug_field='username',
      template_name='kegweb/drinker_detail.html',
      template_object_name='drinker')

def user_detail_by_id(request, user_id):
  try:
    user = models.User.objects.get(pk=user_id)
  except models.User.DoesNotExist:
    raise Http404
  return redirect_to(request, url='/drinker/'+user.username)

def keg_list(request):
  all_kegs = request.kbsite.kegs.all().order_by('-id')
  return object_list(request,
      queryset=all_kegs,
      template_object_name='keg',
      template_name='kegweb/keg_list.html')

#@cache_page(30)
def keg_detail(request, keg_id):
  keg = get_object_or_404(models.Keg, site=request.kbsite, seqn=keg_id)
  context = RequestContext(request, {'keg': keg})
  return render_to_response('kegweb/keg_detail.html', context)

def drink_list(request):
  all_drinks = request.kbsite.drinks.valid()
  return object_list(request,
      queryset=all_drinks,
      template_name='kegweb/drink_list.html',
      template_object_name='drink')

def drink_detail(request, drink_id):
  drink = get_object_or_404(models.Drink, site=request.kbsite, seqn=drink_id)
  context = RequestContext(request, {'drink': drink})
  return render_to_response('kegweb/drink_detail.html', context)

### auth

def webauth(request):
  context = {}
  return render_to_response('kegweb/webauth.html', context)

@login_required
def claim_token(request):
  if request.method == 'POST':
    form = forms.ClaimTokenForm(request.POST)

    if form.is_valid():
      user = form.cleaned_data['user']
      token = form.cleaned_data['token']
      # TODO(mikey): non-superusers should only be able to claim tokens for
      # their own account.
      token.user = user
      token.save()
  else:
    form = forms.ClaimTokenForm()

  context = RequestContext(request)
  context['form'] = form
  return render_to_response('kegweb/claim_token.html', context)




from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import Http404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.generic.simple import redirect_to

@login_required
def account_settings(request):
  context = RequestContext(request)
  profile = request.user.get_profile().TwitterProfile()
  if not profile:
    return render_to_response('contrib/twitter/link.html', context)
  return render_to_response('contrib/twitter/settings.html', context)



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls.defaults import *

urlpatterns = patterns('pykeg.web.api.views',

    url(r'^auth-token/(?P<auth_device>\w+)\.(?P<token_value>\w+)/?$',
        'get_auth_token'),
    url(r'^drink/?$', 'all_drinks'),
    url(r'^drink/(?P<drink_id>\d+)/?$', 'get_drink'),
    url(r'^keg/?$', 'all_kegs'),
    url(r'^keg/(?P<keg_id>\d+)/?$', 'get_keg'),
    url(r'^keg/(?P<keg_id>\d+)/drinks/?$', 'get_keg_drinks'),
    url(r'^tap/?$', 'all_taps'),
    url(r'^tap/(?P<tap_id>[\w\.]+)/?$', 'tap_detail'),
    url(r'^thermo-sensor/?$', 'all_thermo_sensors'),
    url(r'^thermo-sensor/(?P<sensor_name>[^/]+)/?$', 'get_thermo_sensor'),
    url(r'^thermo-sensor/(?P<sensor_name>[^/]+)/logs/?$', 'get_thermo_sensor_logs'),
    url(r'^user/(?P<username>\w+)/?$', 'get_user'),
    url(r'^user/(?P<username>\w+)/drinks/?$', 'get_user_drinks'),

    url(r'^last-drink-id/?$', 'last_drink_id'),
    url(r'^last-drinks/?$', 'last_drinks'),
    url(r'^last-drinks-html/?$', 'last_drinks_html'),

    url(r'^get-access-token/?$', 'get_access_token'),

    url(r'', 'default_handler'),

)



import os.path

from pykeg.core import features

from django.conf import settings
from django.conf.urls.defaults import *
from django.contrib.auth.views import password_reset
from django.contrib.auth.views import password_reset_done
from django.contrib.auth.views import password_reset_confirm
from django.contrib.auth.views import password_reset_complete
from django.contrib import admin
admin.autodiscover()

def basedir():
  """ Get the pwd of this module, eg for use setting absolute paths """
  return os.path.abspath(os.path.dirname(__file__))

urlpatterns = patterns('',
    ### django admin site
    (r'^admin/(.*)', admin.site.root),

    ### static media
    url(r'^site_media/(.*)$',
      'django.views.static.serve',
      {'document_root': os.path.join(basedir(), 'media')},
      name='site-media'),

    url(r'^media/(.*)$',
     'django.views.static.serve',
     {'document_root': settings.MEDIA_ROOT, 'show_indexes': True},
     name='media'),

    ### RESTful api
    (r'^api/', include('pykeg.web.api.urls')),

    ### account
    (r'^account/', include('pykeg.web.account.urls')),
    (r'^accounts/password/reset/$', password_reset, {'template_name':
     'registration/password_reset.html'}),
    (r'^accounts/password/reset/done/$', password_reset_done, {'template_name':
     'registration/password_reset_done.html'}),
    (r'^accounts/password/reset/confirm/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$', password_reset_confirm, {'template_name':
     'registration/password_reset_confirm.html'}),
    (r'^accounts/password/reset/complete/$', password_reset_complete, {'template_name':
     'registration/password_reset_complete.html'}),

    ### kegadmin
    (r'^kegadmin/', include('pykeg.web.kegadmin.urls')),

    ### socialregistration
    (r'^sr/', include('socialregistration.urls')),

    ### main kegweb urls
    (r'', include('pykeg.web.kegweb.urls')),
)

if features.use_facebook():
  urlpatterns += patterns('',
      ### facebook kegweb stuff
      (r'^fb/', include('pykeg.web.contrib.facebook.urls')),
  )

if features.use_twitter():
  urlpatterns += patterns('',
      ### twitter kegweb stuff
      (r'^twitter/', include('pykeg.web.contrib.twitter.urls')),
  )




# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)


DESCRIPTOR = descriptor.FileDescriptor(
  name='kegnet.proto',
  package='',
  serialized_pb='\n\x0ckegnet.proto\"\x06\n\x04Ping\"\x0b\n\tQuitEvent\"\x14\n\x12StartCompleteEvent\"0\n\x0bMeterUpdate\x12\x10\n\x08tap_name\x18\x01 \x02(\t\x12\x0f\n\x07reading\x18\x02 \x02(\x02\"\x88\x02\n\nFlowUpdate\x12\x0f\n\x07\x66low_id\x18\x01 \x02(\r\x12\x10\n\x08tap_name\x18\x02 \x02(\t\x12$\n\x05state\x18\x03 \x02(\x0e\x32\x15.FlowUpdate.FlowState\x12\x10\n\x08username\x18\x04 \x01(\t\x12\x12\n\nstart_time\x18\x05 \x02(\r\x12\x1a\n\x12last_activity_time\x18\x06 \x02(\r\x12\r\n\x05ticks\x18\x07 \x02(\r\x12\x11\n\tvolume_ml\x18\x08 \x02(\x02\"M\n\tFlowState\x12\x0b\n\x07INITIAL\x10\x01\x12\n\n\x06\x41\x43TIVE\x10\x02\x12\x08\n\x04IDLE\x10\x03\x12\x0e\n\nCLOSE_WAIT\x10\x04\x12\r\n\tCOMPLETED\x10\x05\" \n\x0cTapIdleEvent\x12\x10\n\x08tap_name\x18\x01 \x02(\t\"\x80\x01\n\x11\x44rinkCreatedEvent\x12\x0f\n\x07\x66low_id\x18\x01 \x02(\r\x12\x10\n\x08\x64rink_id\x18\x02 \x02(\r\x12\x10\n\x08tap_name\x18\x03 \x02(\t\x12\x12\n\nstart_time\x18\x04 \x02(\r\x12\x10\n\x08\x65nd_time\x18\x05 \x02(\r\x12\x10\n\x08username\x18\x06 \x01(\t\"\xa3\x01\n\x0eTokenAuthEvent\x12\x10\n\x08tap_name\x18\x01 \x02(\t\x12\x18\n\x10\x61uth_device_name\x18\x02 \x02(\t\x12\x13\n\x0btoken_value\x18\x03 \x02(\t\x12*\n\x06status\x18\x04 \x02(\x0e\x32\x1a.TokenAuthEvent.TokenState\"$\n\nTokenState\x12\t\n\x05\x41\x44\x44\x45\x44\x10\x01\x12\x0b\n\x07REMOVED\x10\x02\"\x82\x01\n\rUserAuthEvent\x12\x10\n\x08tap_name\x18\x01 \x02(\t\x12\x11\n\tuser_name\x18\x02 \x02(\t\x12\'\n\x05state\x18\x03 \x02(\x0e\x32\x18.UserAuthEvent.UserState\"#\n\tUserState\x12\t\n\x05\x41\x44\x44\x45\x44\x10\x01\x12\x0b\n\x07REMOVED\x10\x02\"8\n\x0bThermoEvent\x12\x13\n\x0bsensor_name\x18\x01 \x02(\t\x12\x14\n\x0csensor_value\x18\x02 \x02(\x02\"\x81\x01\n\x0b\x46lowRequest\x12\x10\n\x08tap_name\x18\x01 \x02(\t\x12$\n\x07request\x18\x02 \x02(\x0e\x32\x13.FlowRequest.Action\":\n\x06\x41\x63tion\x12\x0e\n\nSTART_FLOW\x10\x01\x12\r\n\tSTOP_FLOW\x10\x02\x12\x11\n\rREPORT_STATUS\x10\x03\"\x16\n\x14HeartbeatSecondEvent\"\x16\n\x14HeartbeatMinuteEvent\"\x14\n\x12HeartbeatHourEvent\"4\n\x10\x43reditAddedEvent\x12\x0e\n\x06\x61mount\x18\x01 \x02(\x02\x12\x10\n\x08username\x18\x02 \x01(\t')



_FLOWUPDATE_FLOWSTATE = descriptor.EnumDescriptor(
  name='FlowState',
  full_name='FlowUpdate.FlowState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='INITIAL', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ACTIVE', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='IDLE', index=2, number=3,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='CLOSE_WAIT', index=3, number=4,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='COMPLETED', index=4, number=5,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=297,
  serialized_end=374,
)

_TOKENAUTHEVENT_TOKENSTATE = descriptor.EnumDescriptor(
  name='TokenState',
  full_name='TokenAuthEvent.TokenState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='ADDED', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='REMOVED', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=669,
  serialized_end=705,
)

_USERAUTHEVENT_USERSTATE = descriptor.EnumDescriptor(
  name='UserState',
  full_name='UserAuthEvent.UserState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='ADDED', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='REMOVED', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=803,
  serialized_end=838,
)

_FLOWREQUEST_ACTION = descriptor.EnumDescriptor(
  name='Action',
  full_name='FlowRequest.Action',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='START_FLOW', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='STOP_FLOW', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='REPORT_STATUS', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=970,
  serialized_end=1028,
)


_PING = descriptor.Descriptor(
  name='Ping',
  full_name='Ping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=16,
  serialized_end=22,
)


_QUITEVENT = descriptor.Descriptor(
  name='QuitEvent',
  full_name='QuitEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=24,
  serialized_end=35,
)


_STARTCOMPLETEEVENT = descriptor.Descriptor(
  name='StartCompleteEvent',
  full_name='StartCompleteEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=37,
  serialized_end=57,
)


_METERUPDATE = descriptor.Descriptor(
  name='MeterUpdate',
  full_name='MeterUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tap_name', full_name='MeterUpdate.tap_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='reading', full_name='MeterUpdate.reading', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=59,
  serialized_end=107,
)


_FLOWUPDATE = descriptor.Descriptor(
  name='FlowUpdate',
  full_name='FlowUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='flow_id', full_name='FlowUpdate.flow_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='tap_name', full_name='FlowUpdate.tap_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='state', full_name='FlowUpdate.state', index=2,
      number=3, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='username', full_name='FlowUpdate.username', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='start_time', full_name='FlowUpdate.start_time', index=4,
      number=5, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='last_activity_time', full_name='FlowUpdate.last_activity_time', index=5,
      number=6, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='ticks', full_name='FlowUpdate.ticks', index=6,
      number=7, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='volume_ml', full_name='FlowUpdate.volume_ml', index=7,
      number=8, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FLOWUPDATE_FLOWSTATE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=110,
  serialized_end=374,
)


_TAPIDLEEVENT = descriptor.Descriptor(
  name='TapIdleEvent',
  full_name='TapIdleEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tap_name', full_name='TapIdleEvent.tap_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=376,
  serialized_end=408,
)


_DRINKCREATEDEVENT = descriptor.Descriptor(
  name='DrinkCreatedEvent',
  full_name='DrinkCreatedEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='flow_id', full_name='DrinkCreatedEvent.flow_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='drink_id', full_name='DrinkCreatedEvent.drink_id', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='tap_name', full_name='DrinkCreatedEvent.tap_name', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='start_time', full_name='DrinkCreatedEvent.start_time', index=3,
      number=4, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='end_time', full_name='DrinkCreatedEvent.end_time', index=4,
      number=5, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='username', full_name='DrinkCreatedEvent.username', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=411,
  serialized_end=539,
)


_TOKENAUTHEVENT = descriptor.Descriptor(
  name='TokenAuthEvent',
  full_name='TokenAuthEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tap_name', full_name='TokenAuthEvent.tap_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='auth_device_name', full_name='TokenAuthEvent.auth_device_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='token_value', full_name='TokenAuthEvent.token_value', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='status', full_name='TokenAuthEvent.status', index=3,
      number=4, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TOKENAUTHEVENT_TOKENSTATE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=542,
  serialized_end=705,
)


_USERAUTHEVENT = descriptor.Descriptor(
  name='UserAuthEvent',
  full_name='UserAuthEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tap_name', full_name='UserAuthEvent.tap_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='user_name', full_name='UserAuthEvent.user_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='state', full_name='UserAuthEvent.state', index=2,
      number=3, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _USERAUTHEVENT_USERSTATE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=708,
  serialized_end=838,
)


_THERMOEVENT = descriptor.Descriptor(
  name='ThermoEvent',
  full_name='ThermoEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='sensor_name', full_name='ThermoEvent.sensor_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='sensor_value', full_name='ThermoEvent.sensor_value', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=840,
  serialized_end=896,
)


_FLOWREQUEST = descriptor.Descriptor(
  name='FlowRequest',
  full_name='FlowRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tap_name', full_name='FlowRequest.tap_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='request', full_name='FlowRequest.request', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FLOWREQUEST_ACTION,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=899,
  serialized_end=1028,
)


_HEARTBEATSECONDEVENT = descriptor.Descriptor(
  name='HeartbeatSecondEvent',
  full_name='HeartbeatSecondEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1030,
  serialized_end=1052,
)


_HEARTBEATMINUTEEVENT = descriptor.Descriptor(
  name='HeartbeatMinuteEvent',
  full_name='HeartbeatMinuteEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1054,
  serialized_end=1076,
)


_HEARTBEATHOUREVENT = descriptor.Descriptor(
  name='HeartbeatHourEvent',
  full_name='HeartbeatHourEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1078,
  serialized_end=1098,
)


_CREDITADDEDEVENT = descriptor.Descriptor(
  name='CreditAddedEvent',
  full_name='CreditAddedEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='amount', full_name='CreditAddedEvent.amount', index=0,
      number=1, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='username', full_name='CreditAddedEvent.username', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1100,
  serialized_end=1152,
)


_FLOWUPDATE.fields_by_name['state'].enum_type = _FLOWUPDATE_FLOWSTATE
_FLOWUPDATE_FLOWSTATE.containing_type = _FLOWUPDATE;
_TOKENAUTHEVENT.fields_by_name['status'].enum_type = _TOKENAUTHEVENT_TOKENSTATE
_TOKENAUTHEVENT_TOKENSTATE.containing_type = _TOKENAUTHEVENT;
_USERAUTHEVENT.fields_by_name['state'].enum_type = _USERAUTHEVENT_USERSTATE
_USERAUTHEVENT_USERSTATE.containing_type = _USERAUTHEVENT;
_FLOWREQUEST.fields_by_name['request'].enum_type = _FLOWREQUEST_ACTION
_FLOWREQUEST_ACTION.containing_type = _FLOWREQUEST;

class Ping(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PING
  
  # @@protoc_insertion_point(class_scope:Ping)

class QuitEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _QUITEVENT
  
  # @@protoc_insertion_point(class_scope:QuitEvent)

class StartCompleteEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _STARTCOMPLETEEVENT
  
  # @@protoc_insertion_point(class_scope:StartCompleteEvent)

class MeterUpdate(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _METERUPDATE
  
  # @@protoc_insertion_point(class_scope:MeterUpdate)

class FlowUpdate(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _FLOWUPDATE
  
  # @@protoc_insertion_point(class_scope:FlowUpdate)

class TapIdleEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TAPIDLEEVENT
  
  # @@protoc_insertion_point(class_scope:TapIdleEvent)

class DrinkCreatedEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _DRINKCREATEDEVENT
  
  # @@protoc_insertion_point(class_scope:DrinkCreatedEvent)

class TokenAuthEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TOKENAUTHEVENT
  
  # @@protoc_insertion_point(class_scope:TokenAuthEvent)

class UserAuthEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _USERAUTHEVENT
  
  # @@protoc_insertion_point(class_scope:UserAuthEvent)

class ThermoEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _THERMOEVENT
  
  # @@protoc_insertion_point(class_scope:ThermoEvent)

class FlowRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _FLOWREQUEST
  
  # @@protoc_insertion_point(class_scope:FlowRequest)

class HeartbeatSecondEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _HEARTBEATSECONDEVENT
  
  # @@protoc_insertion_point(class_scope:HeartbeatSecondEvent)

class HeartbeatMinuteEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _HEARTBEATMINUTEEVENT
  
  # @@protoc_insertion_point(class_scope:HeartbeatMinuteEvent)

class HeartbeatHourEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _HEARTBEATHOUREVENT
  
  # @@protoc_insertion_point(class_scope:HeartbeatHourEvent)

class CreditAddedEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CREDITADDEDEVENT
  
  # @@protoc_insertion_point(class_scope:CreditAddedEvent)

# @@protoc_insertion_point(module_scope)



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Routines from converting data to and from Protocol Buffer format."""

import sys
import time

from pykeg.beerdb import models as bdb_models
from pykeg.core import models
from pykeg.core import models_pb2

if sys.version_info[:2] < (2, 6):
  import simplejson as json
else:
  import json

_CONVERSION_MAP = {}

def converts(kind):
  def decorate(f):
    global _CONVERSION_MAP
    _CONVERSION_MAP[kind] = f
    return f
  return decorate

def time_to_int(timeval):
  return int(time.mktime(timeval.timetuple()))

def ToProto(obj, full=True):
  """Converts the object to protocol buffer format.

  `obj` may be of any object with a protobuf equivalent.
  """
  if obj is None:
    return None
  kind = obj.__class__
  if hasattr(obj, '__iter__'):
    return (ToProto(item, full) for item in obj)
  elif kind in _CONVERSION_MAP:
    return _CONVERSION_MAP[kind](obj, full)
  else:
    raise ValueError, "Unknown object type: %s" % kind

### Model conversions

@converts(models.AuthenticationToken)
def AuthTokenToProto(record, full=True):
  ret = models_pb2.AuthenticationToken()
  ret.id = '%s|%s' % (record.auth_device, record.token_value)
  ret.auth_device = record.auth_device
  ret.token_value = record.token_value
  if record.user:
    ret.username = record.user.username
    if full:
      ret.user.MergeFrom(ToProto(record.user))
  ret.created_time = time_to_int(record.created)
  if record.expires:
    ret.expire_time = time_to_int(record.expires)
  ret.enabled = record.enabled
  if record.pin:
    ret.pin = record.pin
  return ret

@converts(bdb_models.BeerType)
def BeerTypeToProto(beertype, full=True):
  ret = models_pb2.BeerType()
  ret.id = beertype.id
  ret.name = beertype.name
  ret.brewer_id = beertype.brewer.id
  if full:
    ret.brewer.MergeFrom(ToProto(beertype.brewer))
  ret.style_id = beertype.style.id
  if beertype.edition is not None:
    ret.edition = beertype.edition
  if beertype.abv is not None:
    ret.abv = beertype.abv
  if beertype.calories_oz is not None:
    ret.calories_oz = beertype.calories_oz
  if beertype.carbs_oz is not None:
    ret.carbs_oz = beertype.carbs_oz
  return ret

@converts(bdb_models.Brewer)
def BrewerToProto(brewer, full=True):
  ret = models_pb2.Brewer()
  ret.id = brewer.id
  ret.name = brewer.name
  if brewer.country is not None:
    ret.country = brewer.country
  if brewer.origin_state is not None:
    ret.origin_state = brewer.origin_state
  if brewer.origin_city is not None:
    ret.origin_city = brewer.origin_city
  ret.production = brewer.production
  if brewer.url is not None:
    ret.url = brewer.url
  if brewer.description is not None:
    ret.description = brewer.description
  return ret

@converts(bdb_models.BeerStyle)
def BeerStyleToProto(style, full=True):
  ret = models_pb2.BeerStyle()
  ret.id = style.id
  ret.name = style.name
  return ret

@converts(models.Drink)
def DrinkToProto(drink, full=True):
  ret = models_pb2.Drink()
  ret.id = drink.seqn
  ret.ticks = drink.ticks
  ret.volume_ml = drink.volume_ml
  ret.session_id = str(drink.session.seqn)
  ret.pour_time = time_to_int(drink.endtime)
  if drink.duration is not None:
    ret.duration = drink.duration
  ret.is_valid = (drink.status == 'valid')
  if drink.keg:
    ret.keg_id = drink.keg.seqn
    if full:
      ret.keg.MergeFrom(ToProto(drink.keg))
  if drink.user:
    ret.user_id = drink.user.username
    if full:
      ret.user.MergeFrom(ToProto(drink.user))
  if drink.auth_token:
    ret.auth_token = drink.auth_token
  return ret

@converts(models.Keg)
def KegToProto(keg, full=True):
  ret = models_pb2.Keg()
  ret.id = keg.seqn
  ret.type_id = keg.type.id
  if full:
    ret.type.MergeFrom(ToProto(keg.type))
  ret.size_id = keg.size.id
  rem = float(keg.remaining_volume())
  ret.volume_ml_remain = rem
  ret.percent_full = rem / float(keg.full_volume())
  ret.started_time = time_to_int(keg.startdate)
  ret.finished_time = time_to_int(keg.enddate)
  ret.status = keg.status
  if keg.description:
    ret.description = keg.description
  return ret

@converts(models.KegSize)
def KegSizeToProto(size, full=True):
  ret = models_pb2.KegSize()
  ret.id = size.id
  ret.name = size.name
  ret.volume_ml = size.volume_ml
  return ret

@converts(models.KegTap)
def KegTapToProto(tap, full=True):
  ret = models_pb2.KegTap()
  ret.id = str(tap.seqn)
  ret.name = tap.name
  ret.meter_name = tap.meter_name
  ret.ml_per_tick = tap.ml_per_tick
  if tap.description:
    ret.description = tap.description
  if tap.current_keg:
    ret.current_keg_id = tap.current_keg.seqn
    if full:
      ret.current_keg.MergeFrom(ToProto(tap.current_keg))
  if tap.temperature_sensor:
    ret.thermo_sensor_id = str(tap.temperature_sensor.seqn)
    if full:
      log = tap.temperature_sensor.LastLog()
      if log:
        ret.last_temperature.MergeFrom(ToProto(log))
  return ret

@converts(models.DrinkingSession)
def SessionToProto(record, full=True):
  ret = models_pb2.Session()
  ret.id = str(record.seqn)
  ret.start_time = time_to_int(record.starttime)
  ret.end_time = time_to_int(record.endtime)
  ret.volume_ml = record.volume
  return ret

@converts(models.Thermolog)
def ThermoLogToProto(record, full=True):
  ret = models_pb2.ThermoLog()
  ret.id = str(record.seqn)
  ret.sensor_id = str(record.sensor.seqn)
  ret.temperature_c = record.temp
  ret.record_time = time_to_int(record.time)
  return ret

@converts(models.ThermoSensor)
def ThermoSensorToProto(record, full=True):
  ret = models_pb2.ThermoSensor()
  ret.sensor_name = record.raw_name
  ret.nice_name = record.nice_name
  return ret

@converts(models.User)
def UserToProto(user, full=True):
  ret = models_pb2.User()
  ret.username = user.username
  ret.mugshot_url = user.get_profile().MugshotUrl()
  ret.is_active = user.is_active
  ret.is_staff = user.is_staff
  ret.is_superuser = user.is_superuser
  ret.joined_time = time_to_int(user.date_joined)
  return ret

@converts(models.SessionChunk)
def SessionChunkToProto(record, full=True):
  ret = models_pb2.SessionChunk()
  ret.id = str(record.seqn)
  ret.session_id = str(record.session.seqn)
  ret.username = record.user.username
  ret.keg_id = record.keg.seqn
  ret.start_time = time_to_int(record.starttime)
  ret.end_time = time_to_int(record.endtime)
  ret.volume_ml = record.volume_ml
  return ret




# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import os
import random

from django.db import models
from pykeg.core import fields as core_fields
from django_extensions.db import fields as ext_fields
from imagekit.models import ImageModel

PRODUCTION_CHOICES = (
  ('commercial', 'Commercial brewer'),
  ('homebrew', 'Home brewer'),
)

class BeerDBModel(models.Model):
  class Meta:
    abstract = True
  id = ext_fields.UUIDField(auto=True, primary_key=True)
  revision = models.IntegerField(default=0, editable=False)
  added = models.DateTimeField(default=datetime.datetime.now, editable=False)
  edited = models.DateTimeField(editable=False)

  def save(self, *args, **kwargs):
    self.revision += 1
    self.edited = datetime.datetime.now()
    super(BeerDBModel, self).save(*args, **kwargs)


def beer_file_name(instance, filename):
  rand_salt = random.randrange(0xffff)
  new_filename = '%04x-%s' % (rand_salt, filename)
  return os.path.join('beerdb', new_filename)


class BeerDBImageModel(ImageModel, BeerDBModel):
  class Meta:
    abstract = True

class BeerImage(BeerDBImageModel):
  class IKOptions:
    spec_module = 'pykeg.beerdb.imagespecs'
    #cache_dir = 'cache'
    image_field = 'original_image'
    save_count_as = 'num_views'
  original_image = models.ImageField(upload_to=beer_file_name)
  num_views = models.PositiveIntegerField(editable=False, default=0)

  def __str__(self):
    return '%s' % self.original_image.name


class Brewer(BeerDBModel):
  """Describes a producer of beer."""
  name = models.CharField(max_length=255,
      help_text='Name of the brewer')
  country = core_fields.CountryField(default='USA',
      help_text='Country of origin')
  origin_state = models.CharField(max_length=128,
      default='', blank=True, null=True,
      help_text='State of origin, if applicable')
  origin_city = models.CharField(max_length=128, default='', blank=True,
      null=True,
      help_text='City of origin, if known')
  production = models.CharField(max_length=128, choices=PRODUCTION_CHOICES,
      default='commercial')
  url = models.URLField(verify_exists=False, default='', blank=True, null=True,
      help_text='Brewer\'s home page')
  description = models.TextField(default='', blank=True, null=True,
      help_text='A short description of the brewer')

  image = models.ForeignKey(BeerImage, blank=True, null=True,
      related_name='brewers')

  def __str__(self):
    return self.name


class BeerStyle(BeerDBModel):
  """Describes a named style of beer (Stout, IPA, etc)"""
  name = models.CharField(max_length=128,
      help_text='Name of the beer style')

  def __str__(self):
    return self.name


class BeerType(BeerDBModel):
  """Describes a specific kind of beer, by name, brewer, and style."""
  name = models.CharField(max_length=255)
  brewer = models.ForeignKey(Brewer)
  style = models.ForeignKey(BeerStyle)

  edition = models.CharField(max_length=255, blank=True, null=True,
      help_text='For seasonal or special edition beers, enter the '
          'year or other edition name')

  abv = models.FloatField(blank=True, null=True,
      help_text='Alcohol by volume, as percentage (0-100), if known')
  calories_oz = models.FloatField(blank=True, null=True,
      help_text='Calories per ounce of beer, if known')
  carbs_oz = models.FloatField(blank=True, null=True,
      help_text='Carbohydrates per ounce of beer, if known')

  original_gravity = models.FloatField(blank=True, null=True,
      help_text='Original gravity of the beer, if known')
  specific_gravity = models.FloatField(blank=True, null=True,
      help_text='Specific/final gravity of the beer, if known')

  image = models.ForeignKey(BeerImage, blank=True, null=True,
      related_name='beers')

  def __str__(self):
    return "%s by %s" % (self.name, self.brewer)

  def GetImage(self):
    if self.image:
      return self.image
    if self.brewer.image:
      return self.brewer.image
    return None



#!/usr/bin/env python

"""Unittest for units module"""

import unittest
import units

UNITS = units.UNITS

class UnitsTestCase(unittest.TestCase):
  def testReverseMappings(self):
    converter = units.UnitConverter

    # ensure reverse mappings are in place
    for k, orig_val in converter._table.iteritems():
      from_unit, to_unit = k
      reverse_key = (to_unit, from_unit)
      self.assert_(converter.CanConvert(*reverse_key),
                   "reverse for %s not found" % str(k))

      reverse_val = converter._table[reverse_key]
      computed_reverse_val = 1.0/orig_val
      delta = abs(reverse_val - computed_reverse_val)
      # Not using assertAlmostEqual: can we reliably predict the right value of
      # "places"?
      self.assert_(delta < .000001)

  def testSameUnits(self):
    converter = units.UnitConverter
    amt = 123.45
    for unit_type in UNITS:
      self.assertEqual(amt, converter.Convert(amt, unit_type, unit_type))

  def testSimpleConversion(self):
    converter = units.UnitConverter

    self.assertEqual(16, converter.Convert(1, UNITS.Pint, UNITS.Ounce))
    self.assertEqual(8, converter.Convert(1, UNITS.USGallon, UNITS.Pint))
    self.assertNotEqual(1.0, converter.Convert(1, UNITS.Liter, UNITS.Pint))

    self.assertEqual(4, converter.Convert(4000, UNITS.Milliliter, UNITS.Liter))
    self.assertEqual(1.234, converter.Convert(1234, UNITS.Milliliter, UNITS.Liter))

    amt_liter = 1234.56
    amt_gal = converter.Convert(amt_liter, UNITS.Liter, UNITS.USGallon)
    self.assertEqual(amt_liter, converter.Convert(amt_gal, UNITS.USGallon,
                                                  UNITS.Liter))

  def testPartialTable(self):
    """Test a conversion table lacking closure on all units"""
    # fake enum vals
    FOO_UNIT, BAR_UNIT, WEEK_UNIT, DAY_UNIT = (1, 2, 3, 4)
    table = {
        (FOO_UNIT, BAR_UNIT) : 2,  # FOO is twice the size of BAR
        (WEEK_UNIT, DAY_UNIT) : 7.0, # 7 days in a week,
    }
    converter = units._UnitConverter(table)

    # we can convert foos to bars easily
    self.assertEqual(3.0, converter.Convert(1.5, FOO_UNIT, BAR_UNIT))
    self.assertEqual(1.0, converter.Convert(2.0, BAR_UNIT, FOO_UNIT))

    # and days to weeks
    self.assertEqual(14.0, converter.Convert(2.0, WEEK_UNIT, DAY_UNIT))
    self.assertEqual(5.0, converter.Convert(35, DAY_UNIT, WEEK_UNIT))

    # but there's no way to convert BARs to WEEKs
    self.assertRaises(units.ConversionError, converter.Convert, 1.0, BAR_UNIT,
                      WEEK_UNIT)

class QuantityTestCase(unittest.TestCase):
  """Test the units.Quantity class"""
  def testBasicUsage(self):
    v = units.Quantity(1234.0)
    self.assertEqual(1.234, v.ConvertTo.Liter)

    v.IncAmount(1, UNITS.Liter)
    self.assertEqual(2.234, v.ConvertTo.Liter)

    v.Clear()
    self.assertEqual(0, v.ConvertTo.Liter)

    v.SetAmount(256, UNITS.Ounce)
    self.assertEqual(2.0, v.ConvertTo.USGallon)

    v.IncAmount(-0.5, UNITS.USGallon)
    self.assertEqual(1.5, v.ConvertTo.USGallon)

    v1 = units.Quantity(123.0)
    v2 = units.Quantity(123, from_units=units.Quantity.DEFAULT_UNITS)
    self.assertEqual(v1, v2)

  def testOpers(self):
    v1 = units.Quantity(330, UNITS.Milliliter)
    v2 = units.Quantity(1.5, UNITS.Liter)

    # add quantities should work
    res = v1 + v2
    self.assertEqual(res, units.Quantity(1.830, UNITS.Liter))

    # adding int types works like adding same quantity
    res = v1 + 100
    self.assertEqual(res, units.Quantity(0.430, UNITS.Liter))

    # test subtraction
    v3 = v2 - v1
    self.assertEqual(v3, units.Quantity(1170, UNITS.Milliliter))


if __name__ == '__main__':
  unittest.main()



from django.conf.urls.defaults import *

urlpatterns = patterns('pykeg.web.kegadmin.views',
      ### main page
      url(r'^$', 'kegadmin_main', name='kegadmin-main'),
      url(r'^change-kegs/$', 'change_kegs', name='kegadmin-change-kegs'),
      url(r'^edit-taps/$', 'edit_taps', name='kegadmin-edit-taps'),
)




#!/usr/bin/env python
#
# Copyright 2009 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Simple kegboard monitor program.

This app attaches to a kegboard and prints all packets it receives.
"""

from pykeg.core import importhacks

import os

import gflags
import serial

from pykeg.core import kb_app
from pykeg.hw.kegboard import kegboard

FLAGS = gflags.FLAGS


class KegboardMonitorApp(kb_app.App):

  def _SetupSerial(self):
    self._logger.info('Setting up serial port...')
    self._serial_fd = serial.Serial(FLAGS.kegboard_device, FLAGS.kegboard_speed)
    self._reader = kegboard.KegboardReader(self._serial_fd)

  def _MainLoop(self):
    self._SetupSerial()
    self._logger.info('Starting reader loop...')
    ping_message = kegboard.PingCommand()
    self._serial_fd.write(ping_message.ToBytes())
    while not self._do_quit:
      try:
        msg = self._reader.GetNextMessage()
        print msg
      except:
        break
    self._serial_fd.close()
    self._logger.info('Reader loop ended.')


if __name__ == '__main__':
  KegboardMonitorApp.BuildAndRun()



# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
# http://code.google.com/p/protobuf/
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Contains routines for printing protocol messages in text format."""

__author__ = 'kenton@google.com (Kenton Varda)'

import cStringIO
import re

from collections import deque
from google.protobuf.internal import type_checkers
from google.protobuf import descriptor

__all__ = [ 'MessageToString', 'PrintMessage', 'PrintField',
            'PrintFieldValue', 'Merge' ]


# Infinity and NaN are not explicitly supported by Python pre-2.6, and
# float('inf') does not work on Windows (pre-2.6).
_INFINITY = 1e10000    # overflows, thus will actually be infinity.
_NAN = _INFINITY * 0


class ParseError(Exception):
  """Thrown in case of ASCII parsing error."""


def MessageToString(message):
  out = cStringIO.StringIO()
  PrintMessage(message, out)
  result = out.getvalue()
  out.close()
  return result


def PrintMessage(message, out, indent = 0):
  for field, value in message.ListFields():
    if field.label == descriptor.FieldDescriptor.LABEL_REPEATED:
      for element in value:
        PrintField(field, element, out, indent)
    else:
      PrintField(field, value, out, indent)


def PrintField(field, value, out, indent = 0):
  """Print a single field name/value pair.  For repeated fields, the value
  should be a single element."""

  out.write(' ' * indent);
  if field.is_extension:
    out.write('[')
    if (field.containing_type.GetOptions().message_set_wire_format and
        field.type == descriptor.FieldDescriptor.TYPE_MESSAGE and
        field.message_type == field.extension_scope and
        field.label == descriptor.FieldDescriptor.LABEL_OPTIONAL):
      out.write(field.message_type.full_name)
    else:
      out.write(field.full_name)
    out.write(']')
  elif field.type == descriptor.FieldDescriptor.TYPE_GROUP:
    # For groups, use the capitalized name.
    out.write(field.message_type.name)
  else:
    out.write(field.name)

  if field.cpp_type != descriptor.FieldDescriptor.CPPTYPE_MESSAGE:
    # The colon is optional in this case, but our cross-language golden files
    # don't include it.
    out.write(': ')

  PrintFieldValue(field, value, out, indent)
  out.write('\n')


def PrintFieldValue(field, value, out, indent = 0):
  """Print a single field value (not including name).  For repeated fields,
  the value should be a single element."""

  if field.cpp_type == descriptor.FieldDescriptor.CPPTYPE_MESSAGE:
    out.write(' {\n')
    PrintMessage(value, out, indent + 2)
    out.write(' ' * indent + '}')
  elif field.cpp_type == descriptor.FieldDescriptor.CPPTYPE_ENUM:
    out.write(field.enum_type.values_by_number[value].name)
  elif field.cpp_type == descriptor.FieldDescriptor.CPPTYPE_STRING:
    out.write('\"')
    out.write(_CEscape(value))
    out.write('\"')
  elif field.cpp_type == descriptor.FieldDescriptor.CPPTYPE_BOOL:
    if value:
      out.write("true")
    else:
      out.write("false")
  else:
    out.write(str(value))


def Merge(text, message):
  """Merges an ASCII representation of a protocol message into a message.

  Args:
    text: Message ASCII representation.
    message: A protocol buffer message to merge into.

  Raises:
    ParseError: On ASCII parsing problems.
  """
  tokenizer = _Tokenizer(text)
  while not tokenizer.AtEnd():
    _MergeField(tokenizer, message)


def _MergeField(tokenizer, message):
  """Merges a single protocol message field into a message.

  Args:
    tokenizer: A tokenizer to parse the field name and values.
    message: A protocol message to record the data.

  Raises:
    ParseError: In case of ASCII parsing problems.
  """
  message_descriptor = message.DESCRIPTOR
  if tokenizer.TryConsume('['):
    name = [tokenizer.ConsumeIdentifier()]
    while tokenizer.TryConsume('.'):
      name.append(tokenizer.ConsumeIdentifier())
    name = '.'.join(name)

    if not message_descriptor.is_extendable:
      raise tokenizer.ParseErrorPreviousToken(
          'Message type "%s" does not have extensions.' %
          message_descriptor.full_name)
    field = message.Extensions._FindExtensionByName(name)
    if not field:
      raise tokenizer.ParseErrorPreviousToken(
          'Extension "%s" not registered.' % name)
    elif message_descriptor != field.containing_type:
      raise tokenizer.ParseErrorPreviousToken(
          'Extension "%s" does not extend message type "%s".' % (
              name, message_descriptor.full_name))
    tokenizer.Consume(']')
  else:
    name = tokenizer.ConsumeIdentifier()
    field = message_descriptor.fields_by_name.get(name, None)

    # Group names are expected to be capitalized as they appear in the
    # .proto file, which actually matches their type names, not their field
    # names.
    if not field:
      field = message_descriptor.fields_by_name.get(name.lower(), None)
      if field and field.type != descriptor.FieldDescriptor.TYPE_GROUP:
        field = None

    if (field and field.type == descriptor.FieldDescriptor.TYPE_GROUP and
        field.message_type.name != name):
      field = None

    if not field:
      raise tokenizer.ParseErrorPreviousToken(
          'Message type "%s" has no field named "%s".' % (
              message_descriptor.full_name, name))

  if field.cpp_type == descriptor.FieldDescriptor.CPPTYPE_MESSAGE:
    tokenizer.TryConsume(':')

    if tokenizer.TryConsume('<'):
      end_token = '>'
    else:
      tokenizer.Consume('{')
      end_token = '}'

    if field.label == descriptor.FieldDescriptor.LABEL_REPEATED:
      if field.is_extension:
        sub_message = message.Extensions[field].add()
      else:
        sub_message = getattr(message, field.name).add()
    else:
      if field.is_extension:
        sub_message = message.Extensions[field]
      else:
        sub_message = getattr(message, field.name)
        sub_message.SetInParent()

    while not tokenizer.TryConsume(end_token):
      if tokenizer.AtEnd():
        raise tokenizer.ParseErrorPreviousToken('Expected "%s".' % (end_token))
      _MergeField(tokenizer, sub_message)
  else:
    _MergeScalarField(tokenizer, message, field)


def _MergeScalarField(tokenizer, message, field):
  """Merges a single protocol message scalar field into a message.

  Args:
    tokenizer: A tokenizer to parse the field value.
    message: A protocol message to record the data.
    field: The descriptor of the field to be merged.

  Raises:
    ParseError: In case of ASCII parsing problems.
    RuntimeError: On runtime errors.
  """
  tokenizer.Consume(':')
  value = None

  if field.type in (descriptor.FieldDescriptor.TYPE_INT32,
                    descriptor.FieldDescriptor.TYPE_SINT32,
                    descriptor.FieldDescriptor.TYPE_SFIXED32):
    value = tokenizer.ConsumeInt32()
  elif field.type in (descriptor.FieldDescriptor.TYPE_INT64,
                      descriptor.FieldDescriptor.TYPE_SINT64,
                      descriptor.FieldDescriptor.TYPE_SFIXED64):
    value = tokenizer.ConsumeInt64()
  elif field.type in (descriptor.FieldDescriptor.TYPE_UINT32,
                      descriptor.FieldDescriptor.TYPE_FIXED32):
    value = tokenizer.ConsumeUint32()
  elif field.type in (descriptor.FieldDescriptor.TYPE_UINT64,
                      descriptor.FieldDescriptor.TYPE_FIXED64):
    value = tokenizer.ConsumeUint64()
  elif field.type in (descriptor.FieldDescriptor.TYPE_FLOAT,
                      descriptor.FieldDescriptor.TYPE_DOUBLE):
    value = tokenizer.ConsumeFloat()
  elif field.type == descriptor.FieldDescriptor.TYPE_BOOL:
    value = tokenizer.ConsumeBool()
  elif field.type == descriptor.FieldDescriptor.TYPE_STRING:
    value = tokenizer.ConsumeString()
  elif field.type == descriptor.FieldDescriptor.TYPE_BYTES:
    value = tokenizer.ConsumeByteString()
  elif field.type == descriptor.FieldDescriptor.TYPE_ENUM:
    # Enum can be specified by a number (the enum value), or by
    # a string literal (the enum name).
    enum_descriptor = field.enum_type
    if tokenizer.LookingAtInteger():
      number = tokenizer.ConsumeInt32()
      enum_value = enum_descriptor.values_by_number.get(number, None)
      if enum_value is None:
        raise tokenizer.ParseErrorPreviousToken(
            'Enum type "%s" has no value with number %d.' % (
                enum_descriptor.full_name, number))
    else:
      identifier = tokenizer.ConsumeIdentifier()
      enum_value = enum_descriptor.values_by_name.get(identifier, None)
      if enum_value is None:
        raise tokenizer.ParseErrorPreviousToken(
            'Enum type "%s" has no value named %s.' % (
                enum_descriptor.full_name, identifier))
    value = enum_value.number
  else:
    raise RuntimeError('Unknown field type %d' % field.type)

  if field.label == descriptor.FieldDescriptor.LABEL_REPEATED:
    if field.is_extension:
      message.Extensions[field].append(value)
    else:
      getattr(message, field.name).append(value)
  else:
    if field.is_extension:
      message.Extensions[field] = value
    else:
      setattr(message, field.name, value)


class _Tokenizer(object):
  """Protocol buffer ASCII representation tokenizer.

  This class handles the lower level string parsing by splitting it into
  meaningful tokens.

  It was directly ported from the Java protocol buffer API.
  """

  _WHITESPACE = re.compile('(\\s|(#.*$))+', re.MULTILINE)
  _TOKEN = re.compile(
      '[a-zA-Z_][0-9a-zA-Z_+-]*|'           # an identifier
      '[0-9+-][0-9a-zA-Z_.+-]*|'            # a number
      '\"([^\"\n\\\\]|\\\\.)*(\"|\\\\?$)|'  # a double-quoted string
      '\'([^\'\n\\\\]|\\\\.)*(\'|\\\\?$)')  # a single-quoted string
  _IDENTIFIER = re.compile('\w+')
  _INTEGER_CHECKERS = [type_checkers.Uint32ValueChecker(),
                       type_checkers.Int32ValueChecker(),
                       type_checkers.Uint64ValueChecker(),
                       type_checkers.Int64ValueChecker()]
  _FLOAT_INFINITY = re.compile('-?inf(inity)?f?', re.IGNORECASE)
  _FLOAT_NAN = re.compile("nanf?", re.IGNORECASE)

  def __init__(self, text_message):
    self._text_message = text_message

    self._position = 0
    self._line = -1
    self._column = 0
    self._token_start = None
    self.token = ''
    self._lines = deque(text_message.split('\n'))
    self._current_line = ''
    self._previous_line = 0
    self._previous_column = 0
    self._SkipWhitespace()
    self.NextToken()

  def AtEnd(self):
    """Checks the end of the text was reached.

    Returns:
      True iff the end was reached.
    """
    return not self._lines and not self._current_line

  def _PopLine(self):
    while not self._current_line:
      if not self._lines:
        self._current_line = ''
        return
      self._line += 1
      self._column = 0
      self._current_line = self._lines.popleft()

  def _SkipWhitespace(self):
    while True:
      self._PopLine()
      match = re.match(self._WHITESPACE, self._current_line)
      if not match:
        break
      length = len(match.group(0))
      self._current_line = self._current_line[length:]
      self._column += length

  def TryConsume(self, token):
    """Tries to consume a given piece of text.

    Args:
      token: Text to consume.

    Returns:
      True iff the text was consumed.
    """
    if self.token == token:
      self.NextToken()
      return True
    return False

  def Consume(self, token):
    """Consumes a piece of text.

    Args:
      token: Text to consume.

    Raises:
      ParseError: If the text couldn't be consumed.
    """
    if not self.TryConsume(token):
      raise self._ParseError('Expected "%s".' % token)

  def LookingAtInteger(self):
    """Checks if the current token is an integer.

    Returns:
      True iff the current token is an integer.
    """
    if not self.token:
      return False
    c = self.token[0]
    return (c >= '0' and c <= '9') or c == '-' or c == '+'

  def ConsumeIdentifier(self):
    """Consumes protocol message field identifier.

    Returns:
      Identifier string.

    Raises:
      ParseError: If an identifier couldn't be consumed.
    """
    result = self.token
    if not re.match(self._IDENTIFIER, result):
      raise self._ParseError('Expected identifier.')
    self.NextToken()
    return result

  def ConsumeInt32(self):
    """Consumes a signed 32bit integer number.

    Returns:
      The integer parsed.

    Raises:
      ParseError: If a signed 32bit integer couldn't be consumed.
    """
    try:
      result = self._ParseInteger(self.token, is_signed=True, is_long=False)
    except ValueError, e:
      raise self._IntegerParseError(e)
    self.NextToken()
    return result

  def ConsumeUint32(self):
    """Consumes an unsigned 32bit integer number.

    Returns:
      The integer parsed.

    Raises:
      ParseError: If an unsigned 32bit integer couldn't be consumed.
    """
    try:
      result = self._ParseInteger(self.token, is_signed=False, is_long=False)
    except ValueError, e:
      raise self._IntegerParseError(e)
    self.NextToken()
    return result

  def ConsumeInt64(self):
    """Consumes a signed 64bit integer number.

    Returns:
      The integer parsed.

    Raises:
      ParseError: If a signed 64bit integer couldn't be consumed.
    """
    try:
      result = self._ParseInteger(self.token, is_signed=True, is_long=True)
    except ValueError, e:
      raise self._IntegerParseError(e)
    self.NextToken()
    return result

  def ConsumeUint64(self):
    """Consumes an unsigned 64bit integer number.

    Returns:
      The integer parsed.

    Raises:
      ParseError: If an unsigned 64bit integer couldn't be consumed.
    """
    try:
      result = self._ParseInteger(self.token, is_signed=False, is_long=True)
    except ValueError, e:
      raise self._IntegerParseError(e)
    self.NextToken()
    return result

  def ConsumeFloat(self):
    """Consumes an floating point number.

    Returns:
      The number parsed.

    Raises:
      ParseError: If a floating point number couldn't be consumed.
    """
    text = self.token
    if re.match(self._FLOAT_INFINITY, text):
      self.NextToken()
      if text.startswith('-'):
        return -_INFINITY
      return _INFINITY

    if re.match(self._FLOAT_NAN, text):
      self.NextToken()
      return _NAN

    try:
      result = float(text)
    except ValueError, e:
      raise self._FloatParseError(e)
    self.NextToken()
    return result

  def ConsumeBool(self):
    """Consumes a boolean value.

    Returns:
      The bool parsed.

    Raises:
      ParseError: If a boolean value couldn't be consumed.
    """
    if self.token == 'true':
      self.NextToken()
      return True
    elif self.token == 'false':
      self.NextToken()
      return False
    else:
      raise self._ParseError('Expected "true" or "false".')

  def ConsumeString(self):
    """Consumes a string value.

    Returns:
      The string parsed.

    Raises:
      ParseError: If a string value couldn't be consumed.
    """
    return unicode(self.ConsumeByteString(), 'utf-8')

  def ConsumeByteString(self):
    """Consumes a byte array value.

    Returns:
      The array parsed (as a string).

    Raises:
      ParseError: If a byte array value couldn't be consumed.
    """
    list = [self._ConsumeSingleByteString()]
    while len(self.token) > 0 and self.token[0] in ('\'', '"'):
      list.append(self._ConsumeSingleByteString())
    return "".join(list)

  def _ConsumeSingleByteString(self):
    """Consume one token of a string literal.

    String literals (whether bytes or text) can come in multiple adjacent
    tokens which are automatically concatenated, like in C or Python.  This
    method only consumes one token.
    """
    text = self.token
    if len(text) < 1 or text[0] not in ('\'', '"'):
      raise self._ParseError('Exptected string.')

    if len(text) < 2 or text[-1] != text[0]:
      raise self._ParseError('String missing ending quote.')

    try:
      result = _CUnescape(text[1:-1])
    except ValueError, e:
      raise self._ParseError(str(e))
    self.NextToken()
    return result

  def _ParseInteger(self, text, is_signed=False, is_long=False):
    """Parses an integer.

    Args:
      text: The text to parse.
      is_signed: True if a signed integer must be parsed.
      is_long: True if a long integer must be parsed.

    Returns:
      The integer value.

    Raises:
      ValueError: Thrown Iff the text is not a valid integer.
    """
    pos = 0
    if text.startswith('-'):
      pos += 1

    base = 10
    if text.startswith('0x', pos) or text.startswith('0X', pos):
      base = 16
    elif text.startswith('0', pos):
      base = 8

    # Do the actual parsing. Exception handling is propagated to caller.
    result = int(text, base)

    # Check if the integer is sane. Exceptions handled by callers.
    checker = self._INTEGER_CHECKERS[2 * int(is_long) + int(is_signed)]
    checker.CheckValue(result)
    return result

  def ParseErrorPreviousToken(self, message):
    """Creates and *returns* a ParseError for the previously read token.

    Args:
      message: A message to set for the exception.

    Returns:
      A ParseError instance.
    """
    return ParseError('%d:%d : %s' % (
        self._previous_line + 1, self._previous_column + 1, message))

  def _ParseError(self, message):
    """Creates and *returns* a ParseError for the current token."""
    return ParseError('%d:%d : %s' % (
        self._line + 1, self._column + 1, message))

  def _IntegerParseError(self, e):
    return self._ParseError('Couldn\'t parse integer: ' + str(e))

  def _FloatParseError(self, e):
    return self._ParseError('Couldn\'t parse number: ' + str(e))

  def NextToken(self):
    """Reads the next meaningful token."""
    self._previous_line = self._line
    self._previous_column = self._column
    if self.AtEnd():
      self.token = ''
      return
    self._column += len(self.token)

    # Make sure there is data to work on.
    self._PopLine()

    match = re.match(self._TOKEN, self._current_line)
    if match:
      token = match.group(0)
      self._current_line = self._current_line[len(token):]
      self.token = token
    else:
      self.token = self._current_line[0]
      self._current_line = self._current_line[1:]
    self._SkipWhitespace()


# text.encode('string_escape') does not seem to satisfy our needs as it
# encodes unprintable characters using two-digit hex escapes whereas our
# C++ unescaping function allows hex escapes to be any length.  So,
# "\0011".encode('string_escape') ends up being "\\x011", which will be
# decoded in C++ as a single-character string with char code 0x11.
def _CEscape(text):
  def escape(c):
    o = ord(c)
    if o == 10: return r"\n"   # optional escape
    if o == 13: return r"\r"   # optional escape
    if o ==  9: return r"\t"   # optional escape
    if o == 39: return r"\'"   # optional escape

    if o == 34: return r'\"'   # necessary escape
    if o == 92: return r"\\"   # necessary escape

    if o >= 127 or o < 32: return "\\%03o" % o # necessary escapes
    return c
  return "".join([escape(c) for c in text])


_CUNESCAPE_HEX = re.compile('\\\\x([0-9a-fA-F]{2}|[0-9a-f-A-F])')


def _CUnescape(text):
  def ReplaceHex(m):
    return chr(int(m.group(0)[2:], 16))
  # This is required because the 'string_escape' encoding doesn't
  # allow single-digit hex escapes (like '\xf').
  result = _CUNESCAPE_HEX.sub(ReplaceHex, text)
  return result.decode('string_escape')



# Pykeg main settings file.
# Note: YOU SHOULD NOT NEED TO EDIT THIS FILE.  Instead, see the instructions in
# common_settings.py.example.

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.humanize',
    'django.contrib.markup',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django_extensions',
    'imagekit',
    'pykeg.beerdb',
    'pykeg.billing',
    'pykeg.contrib.soundserver',
    'pykeg.contrib.twitter',
    'pykeg.core',
    'pykeg.web',
    'pykeg.web.api',
    'pykeg.web.account',
    'pykeg.web.kegweb',
    'registration',
    'socialregistration',
    'south',
    #'django_nose', # must be after south
)

AUTH_PROFILE_MODULE = "core.UserProfile"
LOGIN_REDIRECT_URL = "/account/"

### Kegweb specific stuff

ROOT_URLCONF = 'pykeg.web.urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates".
    # Always use forward slashes, even on Windows.
    "web/templates",
)

SITE_ID = 1

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT.
# Example: "http://media.lawrence.com"
MEDIA_URL = ''

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/admin_media/'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.request',
    'django.contrib.messages.context_processors.messages',
    'pykeg.web.context_processors.enabled_features',
    'pykeg.web.context_processors.kbsite',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'pykeg.web.middleware.KegbotSiteMiddleware',
    'django.middleware.doc.XViewMiddleware',
)

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
)

### django.contrib.messages
MESSAGE_STORAGE = 'django.contrib.messages.storage.fallback.FallbackStorage'

### django-registration
ACCOUNT_ACTIVATION_DAYS = 3

# Bogus default values (to prevent djangofb from choking if unavailable);
# replace with site-specific values in common_settings.py, if desired.
FACEBOOK_API_KEY = ''
FACEBOOK_SECRET_KEY = ''

### Kegbot

# Number of last drinks to show on the main page.
KEGWEB_LAST_DRINKS_COUNT = 10

### Twitter

TWITTER_CONSUMER_KEY = ''
TWITTER_CONSUMER_SECRET_KEY =''
TWITTER_REQUEST_TOKEN_URL = 'https://api.twitter.com/oauth/request_token'
TWITTER_ACCESS_TOKEN_URL = 'https://api.twitter.com/oauth/access_token'
TWITTER_AUTHORIZATION_URL = 'https://api.twitter.com/oauth/authorize'

#TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'

try:
  import common_settings
  from common_settings import *
except ImportError:
  print 'Error: Could not find common_settings.py'
  print 'Most likely, this means kegbot has not been configured properly.'
  print 'Consult setup documentation.  Exiting...'
  import sys
  sys.exit(1)

### Optional stuff
if FACEBOOK_API_KEY and FACEBOOK_SECRET_KEY:
  INSTALLED_APPS += ('pykeg.contrib.facebook',)
  MIDDLEWARE_CLASSES += (
    'socialregistration.middleware.FacebookMiddleware',
  )
  AUTHENTICATION_BACKENDS += (
    'socialregistration.auth.FacebookAuth',
  )

if TWITTER_CONSUMER_KEY and TWITTER_CONSUMER_SECRET_KEY:
  AUTHENTICATION_BACKENDS += (
    'socialregistration.auth.TwitterAuth',
  )



# -*- coding: latin-1 -*-

from south.db import db
from django.db import models
from pykeg.web.kegweb.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'Page'
        db.create_table('kegweb_page', (
            ('id', orm['kegweb.Page:id']),
            ('name', orm['kegweb.Page:name']),
            ('title', orm['kegweb.Page:title']),
            ('author', orm['kegweb.Page:author']),
            ('content', orm['kegweb.Page:content']),
            ('status', orm['kegweb.Page:status']),
            ('markup', orm['kegweb.Page:markup']),
            ('created_on', orm['kegweb.Page:created_on']),
            ('last_modified', orm['kegweb.Page:last_modified']),
        ))
        db.send_create_signal('kegweb', ['Page'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'Page'
        db.delete_table('kegweb_page')
        
    
    
    models = {
        'auth.group': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)"},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'kegweb.page': {
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'created_on': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'markup': ('django.db.models.fields.CharField', [], {'default': "'markdown'", 'max_length': '32'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'published'", 'max_length': '32'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        }
    }
    
    complete_apps = ['kegweb']



from django.conf.urls.defaults import *

try:
  from registration.views import register
  from pykeg.web.kegweb.forms import KegbotRegistrationForm
  USE_DJANGO_REGISTRATION = True
except ImportError:
  USE_DJANGO_REGISTRATION = False

urlpatterns = patterns('pykeg.web.kegweb.views',
      ### main page
      (r'^$', 'index'),

      ### accountpage
      (r'^claim_token/$', 'claim_token'),

      ### all-time stats
      (r'^stats/$', 'system_stats'),
      (r'^leaders/$', 'redirect_to', {'url': '/stats/'}),

      ### keg related
      (r'^kegs/$', 'keg_list'),
      url(r'^kegs/(?P<keg_id>\d+)', 'keg_detail', name='keg'),
      # redirects to the above for compatibility
      (r'^keg/(?P<keg_id>\d+)', 'redirect_to', {'url': '/kegs/%(keg_id)s'}),

      ### drinkers
      (r'^drinkers/$', 'user_list'),
      url(r'^drinkers/(?P<username>\w+)', 'user_detail', name='drinker'),
      (r'^drinkers/(?P<user_id>\d+)', 'user_detail_by_id'),
      # redirects to the above for compatibility
      (r'^drinker/(?P<user_id>\d+)', 'redirect_to', {'url': '/drinkers/%(user_id)s'}),
      (r'^drinker/(?P<username>\w+)', 'redirect_to', {'url': '/drinkers/%(username)s'}),

      ### drink related
      (r'^drinks/$', 'drink_list'),
      url(r'^drinks/(?P<drink_id>\d+)', 'drink_detail', name='drink'),
      # redirects to the above for compatibility
      (r'^drink/(?P<drink_id>\d+)', 'redirect_to', {'url': '/drinks/%(drink_id)s'}),
      (r'^d/(?P<drink_id>\d+)', 'redirect_to', {'url': '/drinks/%(drink_id)s'}),

)

### accounts and registration
# uses the stock django-registration views, except we need to override the
# registration class for acocunt/register
if USE_DJANGO_REGISTRATION:
  from django.contrib.auth import views as auth_views
  urlpatterns += patterns('',
    url(r'^accounts/register/$', register,
      {'form_class':KegbotRegistrationForm},
      name='registration_register',
    ),
   (r'^accounts/', include('registration.urls')),
  )




from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.admin.views.decorators import staff_member_required
from django.template import RequestContext
from django.shortcuts import render_to_response

from pykeg.core import models

from pykeg.web.kegadmin import forms

@staff_member_required
def kegadmin_main(request):
  context = RequestContext(request)
  return render_to_response('kegadmin/index.html', context)

@staff_member_required
def change_kegs(request):
  context = RequestContext(request)
  form = forms.ChangeKegForm()
  if request.method == 'POST':
    form = forms.ChangeKegForm(request.POST)
    if form.is_valid():
      tap = form.cleaned_data['tap']
      current = tap.current_keg
      if current and current.status != 'offline':
        current.status = 'offline'
        current.save

      new_keg = models.Keg()
      new_keg.type = form.cleaned_data['beer_type']
      new_keg.size = form.cleaned_data['keg_size']
      new_keg.status = 'online'
      if form.cleaned_data['description']:
        new_keg.description = form.cleaned_data['description']
      new_keg.save()
      tap.current_keg = new_keg
      tap.save()
      messages.success(request, 'The new keg was activated.')
      form = forms.ChangeKegForm()
  context['change_keg_form'] = form
  return render_to_response('kegadmin/change-kegs.html', context)

@staff_member_required
def edit_taps(request):
  context = RequestContext(request)
  tap_forms = [forms.TapForm(x) for x in models.KegTap.objects.all()]
  context['forms'] = tap_forms
  return render_to_response('kegadmin/edit-taps.html', context)

@staff_member_required
def view_stats(request):
  context = RequestContext(request)
  keg_stats = models.KegStats.objects.all()
